import unittest
from superdesk import text_utils


class WordCountTestCase(unittest.TestCase):
    def test_word_count_whitespace_string(self):
        self.assertEqual(0, text_utils.get_word_count("   "))

    def test_word_count_p_tags(self):
        self.assertEqual(2, text_utils.get_word_count("<p>foo<strong>s</strong></p><p>bar</p>"))
        self.assertEqual(500, text_utils.get_word_count("<p>word</p>" * 500))

    def test_word_count_brs(self):
        self.assertEqual(2, text_utils.get_word_count("<p>foo<br><br>bar</p>"))
        self.assertEqual(2, text_utils.get_word_count("<p>foo<br /><br />bar</p>"))

    def test_word_count_hrs(self):
        self.assertEqual(2, text_utils.get_word_count("<p>foo<br><hr>bar</p>"))
        self.assertEqual(2, text_utils.get_word_count("<p>foo<br /><hr />bar</p>"))

    def test_word_count_ul(self):
        self.assertEqual(
            3,
            text_utils.get_word_count(
                """
            <ul>
                <li>foo</li>
                <li>bar</li>
                <li>baz</li>
                <li></li>
            </ul>
        """
            ),
        )

    def test_word_count_nitf(self):
        self.assertEqual(
            37,
            text_utils.get_word_count(
                """
        <p>2014: Northern Ireland beat <location>Greece</location> 2-0 in <location>Athens</location>
        with goals from <person>Jamie Ward</person> and <person>Kyle Lafferty</person> to boost their
        hopes of qualifying for <money>Euro 2016</money>. <person>Michael O'Neill's</person> side
        sealed their place at the finals in <chron>October 2015</chron>.</p>"""
            ),
        )

    def test_word_count_nitf_2(self):
        self.assertEqual(
            314,
            text_utils.get_word_count(
                """
        <p>Rio Tinto has kept intact its target for iron ore shipments in 2017 after hitting the mid-point
        of its revised guidance range for 2016. </p><p>The world's second largest iron ore exporter shipped
        327.6 million tonnes of iron ore from its Pilbara operations in 2016, in line with the slightly lowered
        full-year guidance of between 325 and 330 million tonnes.</p><p>It expects to ship between 330 to 340
        million tonnes in 2017 from its main mining hub in WA.</p><p>"We have delivered a strong operational
        performance in 2016, underpinned by our drive for efficiency and maximising cash flow," chief executive
        Jean Sebastien Jacques said in a statement.</p><p>"Our disciplined approach remains in place in 2017,
        with the continued focus on productivity, cost reduction and commercial excellence."</p><p>Rio shipped
        87.7 million tonnes of iron ore in the December quarter - up eight per cent from the preceding three
        months - mainly helped by minimal weather disruption.</p><p>Fourth-quarter production was also up four
        per cent from a year ago to 85.5 million tonnes.</p><p>Sales in the quarter exceeded production by 2.2
        million tonnes, primarily through a drawdown on inventories built at the ports in the third quarter,
        the company said.</p><p>The miner also looks to have capitalised on a strong rebound in iron ore prices
        in 2016, saying 80 per cent of its sales were either on the spot market or on current quarter or current
        month average.</p><p>Rio’s copper production rose four per cent from a year ago to 523,000 tonnes, but
        still came in below its guidance range of 535,000 to 565,000 tonnes due to lower-than-expected production
        at its Kennecott mine in the US and no supplies from the Grasberg joint venture in Indonesia.</p><p>It has
        forecast a wide guidance range of 525,000 to 665,000 tonnes for 2017.</p><p>The miner topped production
        forecasts for bauxite and coking coal, while aluminium output jumped 10 per cent in 2016.</p>"""
            ),
        )

    def test_word_count_html(self):
        # If you change the following text, please change it in client too at
        # superdesk-client-core/scripts/apps/authoring/authoring/tests/WordCount.spec.js
        text = """
        <p>This is a test text with numbers (1 000 000 and 1,000,000 and 1.000.000)
        and <strong>compound word (two-done)</strong> and <em>abbreviation (Washington D.C.)</p>
        <p>it should be the same word count as in client and backend</p>"""
        self.assertEqual(32, text_utils.get_word_count(text))

    def test_decode(self):
        """Test decoding with encoding detection"""
        bytes_str = "téstôù".encode("latin-1")
        decoded = text_utils.decode(bytes_str)
        self.assertEqual(decoded, "téstôù")

    def test_get_par_count(self):
        self.assertEqual(
            3,
            text_utils.get_par_count(
                """
        <p>First paragraph</p>
        <p>Second paragraph</p>
        <p>Last paragraph</p>
        """
            ),
        )

        self.assertEqual(
            3,
            text_utils.get_par_count(
                """
        <p><br></p>
        <p>First paragraph</p>
        <p>Second paragraph</p>
        <p></p>
        <p>Last paragraph</p>
        <p>

        </p>
        """
            ),
        )

        self.assertEqual(
            0,
            text_utils.get_par_count(
                """
        <p>

        </p>
        """
            ),
        )

        self.assertEqual(
            0,
            text_utils.get_par_count(
                """
        <div></div>
        """
            ),
        )

        self.assertEqual(0, text_utils.get_par_count(None))



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from datetime import datetime

from bson import ObjectId
from eve.utils import date_to_str

from superdesk.tests import TestCase
from superdesk.celery_app import try_cast, loads
from superdesk.celery_task_utils import mark_task_as_not_running, is_task_running


class CeleryTestCase(TestCase):

    _id = ObjectId("528de7b03b80a13eefc5e610")

    def test_cast_objectid(self):
        self.assertEqual(try_cast(str(self._id)), self._id)

    def test_cast_datetime(self):
        date = datetime(2012, 12, 12, 12, 12, 12, 0)
        with self.app.app_context():
            s = date_to_str(date)
            self.assertEqual(try_cast(s).day, date.day)

    def test_loads_args(self):
        s = b'{"args": [{"_id": "528de7b03b80a13eefc5e610", "_updated": "2014-09-10T14:31:09+0000"}]}'
        o = loads(s)
        self.assertEqual(o["args"][0]["_id"], self._id)
        self.assertIsInstance(o["args"][0]["_updated"], datetime)

    def test_loads_kwargs(self):
        s = b"""{"kwargs": "{}", "pid": 24998, "eta": null}"""
        o = loads(s)
        self.assertEqual({}, o["kwargs"])
        self.assertIsNone(o["eta"])

    def test_loads_lists(self):
        s = b"""[{}, {"foo": null}]"""
        o = loads(s)
        self.assertEqual([{}, {"foo": None}], o)

    def test_loads_zero(self):
        s = b"""[0]"""
        o = loads(s)
        self.assertEqual([0], o)

    def test_loads_boolean(self):
        s = b"""[{"foo": false, "bar": true}]"""
        o = loads(s)
        self.assertEqual([{"foo": False, "bar": True}], o)


class CeleryTaskRaceTest(TestCase):
    def test_the_second_update_fails_if_already_running(self):
        provider = {"_id": "abc", "name": "test provider", "update_schedule": {"minutes": 1}}
        removed = mark_task_as_not_running(provider["name"], provider["_id"])
        self.assertFalse(removed)
        failed_to_mark_as_running = is_task_running(provider["name"], provider["_id"], {"minutes": 1})
        self.assertFalse(failed_to_mark_as_running, "Failed to mark ingest update as running")

        failed_to_mark_as_running = is_task_running(provider["name"], provider["_id"], {"minutes": 1})
        self.assertTrue(failed_to_mark_as_running, "Ingest update marked as running, possible race condition")

        removed = mark_task_as_not_running(provider["name"], provider["_id"])
        self.assertTrue(removed, "Failed to mark ingest update as not running.")



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2019 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from superdesk.resource import Resource
from superdesk.metadata.utils import item_url
from superdesk.auth_server.scopes import Scope


class DesksResource(Resource):
    url = "desks"
    item_url = item_url
    item_methods = ["GET"]
    resource_methods = ["GET"]
    allow_unknown = True
    datasource = {"source": "desks", "default_sort": [("name", 1)]}
    privileges = {"GET": Scope.DESKS_READ.name}



import os
import shutil
import tempfile
from flask import json
from unittest.mock import patch

from apps.prepopulate.app_initialize import AppInitializeWithDataCommand
from apps.prepopulate.app_scaffold_data import AppScaffoldDataCommand
from apps.prepopulate.app_initialize import fillEnvironmentVariables
from superdesk import get_resource_service, app
from superdesk.tests import TestCase


class AppInitializeWithDataCommandTestCase(TestCase):
    def _run(self, *a, **kw):
        command = AppInitializeWithDataCommand()
        return command.run(*a, **kw)

    def test_app_initialization(self):
        result = self._run()
        self.assertEqual(result, 0)

    def test_app_initialization_multiple_loads(self):
        result = self._run()
        self.assertEqual(result, 0)
        result = self._run()
        self.assertEqual(result, 0)

    def data_scaffolding_test(self):
        result = self._run(["desks", "stages"], sample_data=True)
        self.assertEqual(result, 0)

        docs = [
            {
                "_id": str(x),
                "type": "text",
                "abstract": "test abstract {}".format(x),
                "headline": "test headline {}".format(x),
                "body_html": "test long story body {}".format(x),
                "state": "published",
            }
            for x in range(0, 40)
        ]
        get_resource_service("published").post(docs)

        stories_per_desk = 2
        existing_desks = 18
        command = AppScaffoldDataCommand()
        result = command.run(stories_per_desk)
        self.assertEqual(result, 0)

        cursor = get_resource_service("desks").get_from_mongo(None, {})
        self.assertEqual(cursor.count(), existing_desks)

        cursor = get_resource_service("archive").get_from_mongo(None, {})
        self.assertEqual(cursor.count(), existing_desks * stories_per_desk)

    def test_sample_data(self):
        result = self._run(sample_data=True)
        self.assertEqual(result, 0)

        cursor = get_resource_service("desks").get_from_mongo(None, {})
        self.assertEqual(cursor.count(), 18)

    def test_app_initialization_index_creation(self):
        result = self._run()
        self.assertEqual(result, 0)
        result = app.data.mongo.pymongo(resource="archive").db["archive"].index_information()
        self.assertTrue("groups.refs.residRef_1" in result)
        self.assertTrue(result["groups.refs.residRef_1"]["sparse"])

    @patch.dict(os.environ, {"REUTERS_USERNAME": "r_u", "REUTERS_PASSWORD": "r_p"})
    def test_app_initialization_set_env_variables(self):
        item = {"username": "#ENV_REUTERS_USERNAME#", "password": "#ENV_REUTERS_PASSWORD#"}
        crt_item = fillEnvironmentVariables(item)
        self.assertEqual(crt_item["username"], "r_u")
        self.assertEqual(crt_item["password"], "r_p")

    @patch.dict(os.environ, {"REUTERS_USERNAME": "", "REUTERS_PASSWORD": "r_p"})
    def test_app_initialization_notset_env_variables(self):
        item = {"username": "#ENV_REUTERS_USERNAME#", "password": "#ENV_REUTERS_PASSWORD#"}
        crt_item = fillEnvironmentVariables(item)
        self.assertTrue(not crt_item)

    def test_init_keeps_user_modifications(self):
        self._run(["vocabularies"])
        urgency = self.app.data.find_one("vocabularies", req=None, _id="urgency")
        self.assertIsNotNone(urgency)
        self.assertEqual("init", urgency["_etag"])

        updates = {"display_name": "FOO"}
        self.app.data.update("vocabularies", "urgency", updates, urgency)

        self._run(["vocabularies"])
        urgency = self.app.data.find_one("vocabularies", req=None, _id="urgency")
        self.assertEqual("FOO", urgency["display_name"])
        self.assertNotEqual("init", urgency["_etag"])

        self._run(["vocabularies"], None, None, True)
        urgency = self.app.data.find_one("vocabularies", req=None, _id="urgency")
        self.assertEqual("Urgency", urgency["display_name"])
        self.assertEqual("init", urgency["_etag"])

    def test_init_can_combine_files_from_folders(self):
        init_dir = tempfile.mkdtemp("init", "test")
        self.app.config.update({"INIT_DATA_PATH": init_dir})

        with open(os.path.join(init_dir, "vocabularies.json"), "w") as f:
            f.write(json.dumps([{"_id": "foo"}]))
            f.flush()

        self._run(["vocabularies"])
        self.assertEqual(self.app.data.find("vocabularies", req=None, lookup={})[0].count(), 1)

        self._run(["validators"])
        self.assertGreater(self.app.data.find("validators", req=None, lookup={})[0].count(), 0)

        self.app.config.pop("INIT_DATA_PATH", None)
        shutil.rmtree(init_dir)



#!/usr/bin/env python
# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2016 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import superdesk
from superdesk.utils import ListCursor
from flask import current_app as app


class AllowedValuesResource(superdesk.Resource):
    """Resource allowing apps to set allowed values which can be used by client to filter certain options."""

    resource_methods = ["GET"]
    item_methods = []
    schema = {
        "items": {
            "type": "list",
        },
    }


class AllowedValuesService(superdesk.Service):
    def get(self, req, lookup):
        allowed = []
        for resource, config in app.config.get("DOMAIN", {}).items():
            for field, field_config in config.get("schema", {}).items():
                if field_config.get("allowed"):
                    allowed.append(
                        {
                            "_id": "{}.{}".format(resource, field),
                            "items": [str(item) for item in field_config["allowed"]],
                        }
                    )
        return ListCursor(allowed)


def init_app(app):
    AllowedValuesResource("allowed_values", app, AllowedValuesService())



import time
import unittest

from unittest.mock import patch, ANY
from superdesk.timer import timer


class TimerTestCase(unittest.TestCase):
    @patch("superdesk.timer.logger")
    def test_timer(self, logger):
        with timer("foo"):
            time.sleep(0.1)
        logger.info.assert_called_once_with("%s: %.3fms", "foo", ANY)
        self.assertAlmostEqual(0.1 * 1000, logger.info.call_args[0][2], 0)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2018 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import json
from copy import deepcopy
from eve.utils import config, ParsedRequest
from superdesk import get_resource_service
from superdesk.errors import StopDuplication, InvalidStateTransitionError
from superdesk.metadata.item import (
    ITEM_STATE,
    CONTENT_STATE,
    PUBLISH_SCHEDULE,
    SCHEDULE_SETTINGS,
    PROCESSED_FROM,
    PUBLISH_STATES,
)
from apps.archive.common import ITEM_OPERATION
from apps.publish.content.common import publish_services
from apps.content_types import apply_schema
from flask_babel import lazy_gettext


def internal_destination_auto_publish(item, **kwargs):
    """Auto publish the item using internal destination

    :param dict item: item to be published
    :param kwargs:
    :raises StopDuplication: to indicate the superdesk.internal_destination.handle_item_published
    to stop duplication as duplication is handle by this method.
    """
    if item.get(ITEM_STATE) not in PUBLISH_STATES:
        raise InvalidStateTransitionError(
            message="Internal Destination auto publish macro can " "only be called after publishing the item."
        )
    operation = item.get(ITEM_OPERATION)
    archive_action_service = get_resource_service(publish_services.get(operation))
    archive_service = get_resource_service("archive")
    extra_fields = [PUBLISH_SCHEDULE, SCHEDULE_SETTINGS]
    # if any macro is doing publishing then we need the duplicate item that was published earlier
    req = ParsedRequest()
    req.where = json.dumps(
        {"$and": [{PROCESSED_FROM: item.get(config.ID_FIELD)}, {"task.desk": str(item.get("task").get("desk"))}]}
    )
    req.max_results = 1
    overwrite_item = next((archive_service.get_from_mongo(req=req, lookup=None)), None)

    # keep pubslish_schedule and schedule_settings in updates so that state can be set to scheduled
    updates = {PUBLISH_SCHEDULE: item[PUBLISH_SCHEDULE], SCHEDULE_SETTINGS: item[SCHEDULE_SETTINGS]}
    if item.get(ITEM_STATE) == CONTENT_STATE.PUBLISHED or not overwrite_item:
        new_id = archive_service.duplicate_content(item, state="routed", extra_fields=extra_fields)
        updates[ITEM_STATE] = item.get(ITEM_STATE)
        updates[PROCESSED_FROM] = item[config.ID_FIELD]

        get_resource_service("archive_publish").patch(id=new_id, updates=updates)
    else:
        if overwrite_item:
            # get the schema fields
            schema_item = apply_schema(deepcopy(item))
            keys_to_delete = ["source", "unique_id", "unique_name", "original_id", "expiry", "correction_sequence"]
            # remove the keys
            archive_service.remove_after_copy(schema_item, delete_keys=keys_to_delete)
            # get the diff
            updates.update(
                {
                    key: val
                    for key, val in schema_item.items()
                    if overwrite_item.get(key) != val and not key.startswith("_")
                }
            )

            archive_action_service.patch(id=overwrite_item[config.ID_FIELD], updates=updates)

    # raise stop duplication on successful completion so that
    # internal destination superdesk.internal_destination.handle_item_published
    # will not duplicate the item.
    raise StopDuplication()


name = "Internal_Destination_Auto_Publish"
label = lazy_gettext("Internal Destination Auto Publish")
callback = internal_destination_auto_publish
access_type = "backend"
action_type = "direct"



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014, 2015, 2016, 2017 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license
import logging
from datetime import timedelta

from superdesk.tests import TestCase
from superdesk import get_resource_service
from superdesk.utc import utcnow

from ..commands.remove_expired_items import RemoveExpiredItems

logger = logging.getLogger(__name__)

utc = utcnow()
expired = utc - timedelta(days=10)
not_expired = utc - timedelta(days=1)
items = [
    {"_id": "1", "_updated": not_expired, "type": "text"},  # Single item, not expired
    {"_id": "2", "_updated": expired, "type": "text"},  # Single item, expired
    # Evolved from: parent expired, child not expired
    {"_id": "3", "_updated": expired, "type": "text"},
    {"_id": "4", "_updated": not_expired, "type": "text", "evolvedfrom": "3", "ancestors": ["3"]},
    # Evolved from: parent expired, child expired
    {"_id": "5", "_updated": expired, "type": "text"},
    {"_id": "6", "_updated": expired, "type": "text", "evolvedfrom": "5", "ancestors": ["5"]},
    # Multi-branch evolved from,
    {"_id": "7", "_updated": expired, "type": "text"},
    {"_id": "8", "_updated": expired, "type": "text", "evolvedfrom": "7", "ancestors": ["7"]},
    {"_id": "9", "_updated": not_expired, "type": "text", "evolvedfrom": "8", "ancestors": ["7", "8"]},
    # Multi-branch evolved from
    {"_id": "10", "_updated": expired, "type": "text"},
    {"_id": "11", "_updated": expired, "type": "text", "evolvedfrom": "10", "ancestors": ["10"]},
    {"_id": "12", "_updated": expired, "type": "text", "evolvedfrom": "11", "ancestors": ["10", "11"]},
]

can_expire = [
    "2",  # Single item, expired
    "5",
    "6",  # Evolved from, parent expired, child expired
    "10",
    "11",
    "12",  # Multi-branch evolved from
]


class RemoveExpiredItemsTest(TestCase):
    def setUp(self):
        self.app.data.insert("items", items)
        self.command = RemoveExpiredItems()
        self.command.expiry_days = 8
        self.now = utcnow()

    def _get_items(self):
        return list(get_resource_service("items").get_from_mongo(req=None, lookup=None))

    def test_remove_expired_items(self):
        self.command._remove_expired_items(self.now, self.command.expiry_days)
        items = self._get_items()

        self.assertEqual(len(items), 6)
        for item in items:
            self.assertNotIn(item["_id"], can_expire)

    def test_get_expired_chain(self):
        items_service = get_resource_service("items")
        for item in self._get_items():
            if self.command._get_expired_chain(items_service, item, self.now):
                self.assertTrue(item["_id"] in can_expire)
            else:
                self.assertFalse(item["_id"] in can_expire)

    def test_run(self):
        self.command.run(self.command.expiry_days)
        items = self._get_items()

        self.assertEqual(len(items), 6)
        for item in items:
            self.assertNotIn(item["_id"], can_expire)

    def test_has_expired(self):
        has_expired = self.command._has_expired(items[1], self.now)
        self.assertTrue(has_expired)

        has_expired = self.command._has_expired(items[0], self.now)
        self.assertFalse(has_expired)

    def test_get_children(self):
        service = get_resource_service("items")
        children = self.command._get_children(service, items[0])
        self.assertEqual(children, [])

        children = [child["_id"] for child in self.command._get_children(service, items[2])]
        self.assertEqual(children, ["4"])

        children = [child["_id"] for child in self.command._get_children(service, items[9])]
        self.assertEqual(children, ["11", "12"])



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from superdesk.resource import Resource
from superdesk.metadata.item import metadata_schema, not_analyzed
from content_api import MONGO_PREFIX, ELASTIC_PREFIX

code_mapping = {"type": "object", "properties": {"name": not_analyzed, "code": not_analyzed}}

schema = {
    "_id": metadata_schema["_id"],
    "associations": metadata_schema["associations"],
    "anpa_category": metadata_schema["anpa_category"],
    "body_html": {
        "type": "string",
        "mapping": {"type": "string", "analyzer": "html_field_analyzer", "search_analyzer": "standard"},
    },
    "body_text": {"type": "string"},
    "byline": {"type": "string"},
    "copyrightnotice": metadata_schema["copyrightnotice"],
    "copyrightholder": {"type": "string"},
    "description_html": {"type": "string"},
    "description_text": {"type": "string"},
    "headline": {"type": "string"},
    "language": metadata_schema["language"],
    "located": {"type": "string"},
    "mimetype": metadata_schema["mimetype"],
    "organization": {"type": "list"},
    "person": {"type": "list"},
    "place": metadata_schema["place"],
    "profile": {"type": "string"},
    "pubstatus": metadata_schema["pubstatus"],
    "renditions": {"type": "dict"},
    "service": {"type": "list", "mapping": code_mapping},
    "slugline": {"type": "string"},
    "source": metadata_schema["source"],
    "subject": {"type": "list", "mapping": code_mapping},
    "keywords": metadata_schema["keywords"],
    "type": metadata_schema["type"],
    "urgency": {"type": "integer"},
    "priority": {"type": "integer"},
    "uri": metadata_schema["guid"],  # we use guid value for uri, so index it in a same way
    "usageterms": {"type": "string"},
    "version": {"type": "string", "required": True, "empty": False, "nullable": False},
    "versioncreated": {"type": "datetime", "required": True},
    "firstcreated": {"type": "datetime"},
    "firstpublished": {"type": "datetime", "required": False},
    "embargoed": {"type": "datetime"},
    "evolvedfrom": Resource.not_analyzed_field(),
    "nextversion": Resource.not_analyzed_field(),
    "subscribers": Resource.not_analyzed_field("list"),
    "ednote": {"type": "string"},
    "signal": {
        "type": "list",
        "mapping": {
            "type": "object",
            "properties": {"code": not_analyzed, "name": not_analyzed, "scheme": not_analyzed},
        },
    },
    "genre": {"type": "list", "mapping": code_mapping},
    "ancestors": Resource.not_analyzed_field("list"),
    "attachments": {
        "type": "list",
        "schema": {
            "type": "dict",
        },
    },
    "annotations": {
        "type": "list",
        "mapping": {"type": "object", "properties": {"id": not_analyzed, "type": not_analyzed, "body": not_analyzed}},
    },
    "bookmarks": Resource.not_analyzed_field("list"),
    "downloads": Resource.not_analyzed_field("list"),  # list of user ids who downloaded this item
    "shares": Resource.not_analyzed_field("list"),  # list of user ids who shared this item
    "prints": Resource.not_analyzed_field("list"),  # list of user ids who printed this item
    "copies": Resource.not_analyzed_field("list"),  # list of user ids who copied this item
    "extra": metadata_schema["extra"],
    "authors": metadata_schema["authors"],
    "wordcount": metadata_schema["word_count"],
    "charcount": {"type": "integer"},
    "readtime": {"type": "integer"},
    # newsroom agenda links
    "event_id": metadata_schema["guid"],
    "planning_id": metadata_schema["guid"],
    "coverage_id": metadata_schema["guid"],
    "agenda_id": metadata_schema["guid"],
    "agenda_href": metadata_schema["guid"],
}


class ItemsResource(Resource):
    """A class defining and configuring the /items API endpoint."""

    # Example of an ID of an object in database (whitout quotes):
    #
    #     "tag:example.com,0000:newsml_BRE9A605"
    #     "tag:localhost:2015:f4b35e12-559b-4a2b-b1f2-d5e64048bde8"
    #
    item_url = r'regex("[\w,.:-]+")'
    schema = schema

    datasource = {
        "search_backend": "elastic",
        "elastic_filter": {"bool": {"must_not": {"term": {"type": "composite"}}}},
        "default_sort": [("versioncreated", -1)],
    }

    mongo_indexes = {"_ancestors_": [("ancestors", 1)]}

    item_methods = ["GET"]
    resource_methods = ["GET"]
    versioning = True
    mongo_prefix = MONGO_PREFIX
    elastic_prefix = ELASTIC_PREFIX



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import superdesk
from superdesk import get_resource_service
from superdesk.errors import ProviderError


class AddProvider(superdesk.Command):
    """Add ingest provider.

    Example:
    ::

        $ python manage.py ingest:provider --provider='{"update_schedule" : { "minutes" : 5, "seconds" : 0 },
            "idle_time" : { "hours" : 0, "minutes" : 0 }, "content_expiry" : 2880, "name" : "aap-demo",
             "source" : "aap-demo", "feeding_service" : "rss",
             "config" : { "url" : "https://abcnews.go.com/abcnews/primetimeheadlines", "field_aliases" : [ ] },
             "feed_parser" : null, "content_types" : [ "text" ]}'

    """

    option_list = {
        superdesk.Option("--provider", "-p", dest="provider", required=True),
    }

    def run(self, provider):
        try:
            data = {}
            data = superdesk.json.loads(provider)
            data.setdefault("content_expiry", superdesk.app.config["INGEST_EXPIRY_MINUTES"])

            validator = superdesk.app.validator(
                superdesk.app.config["DOMAIN"]["ingest_providers"]["schema"], "ingest_providers"
            )
            validation = validator.validate(data)

            if validation:
                get_resource_service("ingest_providers").post([data])
                return data
            else:
                ex = Exception(
                    "Failed to add Provider as the data provided is invalid. Errors: {}".format(str(validator.errors))
                )
                raise ProviderError.providerAddError(exception=ex, provider=data)
        except Exception as ex:
            raise ProviderError.providerAddError(ex, data)


superdesk.command("ingest:provider", AddProvider())



from flask import abort, request
from superdesk.resource import Resource
from superdesk.services import Service
from superdesk.metadata.utils import item_url
from superdesk.metadata.item import CONTENT_STATE
from .archive import ArchiveResource


ARCHIVE_DATASOURCE = ArchiveResource.datasource


def exclude_filter():
    if request:
        return {
            "bool": {
                "must_not": {
                    "term": {
                        "_id": request.view_args["item_id"],
                    }
                }
            }
        }


class ArchiveRelatedResource(Resource):
    endpoint_name = "archive_related"
    url = "archive/<{0}:item_id>/related".format(item_url)
    schema = {"lock_action": {"type": "string"}}
    datasource = {
        "source": "archive",
        "projection": ARCHIVE_DATASOURCE["projection"],
        "default_sort": ARCHIVE_DATASOURCE["default_sort"],
        "elastic_filter": {
            "bool": {
                "must": {
                    "terms": {
                        "state": [
                            CONTENT_STATE.INGESTED,
                            CONTENT_STATE.ROUTED,
                            CONTENT_STATE.FETCHED,
                            CONTENT_STATE.SUBMITTED,
                            CONTENT_STATE.PROGRESS,
                            CONTENT_STATE.PUBLISHED,
                            CONTENT_STATE.CORRECTED,
                            CONTENT_STATE.SCHEDULED,
                            CONTENT_STATE.KILLED,
                            CONTENT_STATE.RECALLED,
                            CONTENT_STATE.UNPUBLISHED,
                        ]
                    }
                },
            }
        },
        "elastic_filter_callback": exclude_filter,
    }
    resource_methods = ["GET"]
    item_methods = []
    resource_title = endpoint_name


class ArchiveRelatedService(Service):
    def get(self, req, lookup):
        item = self.find_one(req=req, _id=lookup["item_id"])
        if not item or not item.get("family_id"):
            abort(404)
        return super().get(req, {"family_id": item["family_id"]})



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import flask
from flask import request, current_app as app
from flask_babel import _
from eve.utils import config

from superdesk import utils as utils, get_resource_service, get_resource_privileges
from superdesk.services import BaseService
from superdesk.errors import SuperdeskApiError
from superdesk.users.errors import UserInactiveError
from apps import auth
from apps.auth.errors import UserDisabledError


class AuthService(BaseService):
    def authenticate(self, document):
        """Authenticate user according to credentials

        :param documents: credentials for this authentication mechanism
        :return: authenticated user
        :raise: CredentialsAuthError if authentication is invalid
        """
        raise NotImplementedError()

    def on_create(self, docs):
        for doc in docs:
            user = self.authenticate(doc)
            if not user:
                raise ValueError()
            if "is_enabled" in user and not user.get("is_enabled", False):
                raise UserDisabledError()
            if not user.get("is_active", False):
                raise UserInactiveError()
            self.set_auth_default(doc, user["_id"])

    def on_created(self, docs):
        for doc in docs:
            get_resource_service("preferences").set_session_based_prefs(doc["_id"], doc["user"])

    def set_auth_default(self, doc, user_id):
        doc["user"] = user_id
        doc["token"] = utils.get_random_string(40)
        doc.pop("password", None)

    def update_session(self, updates=None):
        """Update current session with given data.

        :param updates: updates to be made
        """
        if not updates:
            updates = {}
        self.system_update(flask.g.auth["_id"], updates, flask.g.auth)

    def on_fetched_item(self, doc: dict) -> None:
        if str(doc["user"]) != str(auth.get_user_id()):
            raise SuperdeskApiError.notFoundError(_("Not found."))

    def on_deleted(self, doc):
        """Runs on delete of a session

        :param doc: A deleted auth doc AKA a session
        :return:
        """
        # notify that the session has ended
        app.on_session_end(doc["user"], doc["_id"])

    def is_authorized(self, **kwargs) -> bool:
        """
        Check auth for intrinsic methods.
        """
        method = kwargs["method"]
        user = auth.get_user()

        # delete token is a part of `users` privelege
        # user with `users` privelege can delete sessions of any user
        # user without `users` privelege can delete only it's own session (logout)
        if method == "DELETE":
            _auth = self.find_one(req=None, _id=kwargs.get("_id"))
            if _auth and _auth.get("user") == user.get("_id"):
                return True

            active_privileges = user.get("active_privileges", {})
            users_resource_privileges = get_resource_privileges("users").get(method, None)
            return active_privileges.get(users_resource_privileges, False)

        return True


class UserSessionClearService(BaseService):
    def delete(self, lookup):
        """Delete user session.

        Deletes all the records from auth and corresponding
        session_preferences from user collections
        If there are any orphan session_preferences exist they get deleted as well
        """
        users_service = get_resource_service("users")
        user_id = request.view_args["user"]
        user = users_service.find_one(req=None, _id=user_id)
        sessions = get_resource_service("auth").get(req=None, lookup={"user": user_id})

        error_message = self.__can_clear_sessions(user)
        if error_message:
            raise SuperdeskApiError.forbiddenError(message=error_message)

        # Delete all the sessions
        for session in sessions:
            get_resource_service("auth").delete_action({config.ID_FIELD: str(session[config.ID_FIELD])})

        # Check if any orphan session_preferences exist for the user
        if user.get("session_preferences"):
            # Delete the orphan sessions
            users_service.patch(user[config.ID_FIELD], {"session_preferences": {}})

        return [{"complete": True}]

    def __can_clear_sessions(self, user):
        """Checks if the session clear request is Invalid.

        Operation is invalid if one of the below is True:
            1. Check if the user exists.
            2. Check if the user is clearing his/her own sessions.

        :return: error message if invalid.
        """

        if not user:
            return "Invalid user to clear sessions."

        if str(user[config.ID_FIELD]) == str(flask.g.user[config.ID_FIELD]):
            return "Not allowed to clear your own sessions."



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from superdesk.resource import Resource
from content_api import MONGO_PREFIX, ELASTIC_PREFIX
from content_api.items.resource import schema


class SearchResource(Resource):

    item_url = 'regex("[\w,.:-]+")'
    schema = schema

    datasource = {
        "search_backend": "elastic",
        "source": "items",
        "elastic_filter": {"bool": {"must_not": {"term": {"type": "composite"}}}},
        "default_sort": [("_updated", -1)],
        "aggregations": {
            "type": {"terms": {"field": "type"}},
            "category": {"terms": {"field": "service.name", "size": 100}},
            "source": {"terms": {"field": "source", "size": 100}},
            "urgency": {"terms": {"field": "urgency"}},
            "priority": {"terms": {"field": "priority"}},
            "genre": {"terms": {"field": "genre.name", "size": 100}},
        },
    }

    item_methods = ["GET"]
    resource_methods = ["GET"]
    mongo_prefix = MONGO_PREFIX
    elastic_prefix = ELASTIC_PREFIX
    privileges = {"GET": "search_capi"}



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import logging
from typing import Dict, Any

from superdesk import get_resource_service
from superdesk.notification import push_notification
from superdesk.resource import Resource
from superdesk.services import BaseService
from superdesk.utils import SuperdeskBaseEnum
from flask import current_app as app

logger = logging.getLogger(__name__)


PUBLISHED_IN_PACKAGE = "published_in_package"


class QueueState(SuperdeskBaseEnum):
    PENDING = "pending"
    IN_PROGRESS = "in-progress"
    RETRYING = "retrying"
    SUCCESS = "success"
    CANCELED = "canceled"
    ERROR = "error"
    FAILED = "failed"


class PublishQueueResource(Resource):
    schema = {
        "item_id": {"type": "string", "required": True},
        "item_version": {"type": "integer", "nullable": False},
        "formatted_item": {"type": "string", "nullable": False},
        "item_encoding": {"type": "string", "nullable": True},
        "encoded_item_id": {"type": "objectid", "nullable": True},
        "subscriber_id": Resource.rel("subscribers"),
        "codes": {"type": "list", "nullable": True},
        "destination": {
            "type": "dict",
            "schema": {
                "name": {"type": "string", "required": True, "empty": False},
                "format": {"type": "string", "required": True},
                "delivery_type": {"type": "string", "required": True},
                "config": {"type": "dict"},
            },
        },
        PUBLISHED_IN_PACKAGE: {"type": "string"},
        "published_seq_num": {"type": "integer"},
        # publish_schedule is to indicate the item schedule datetime.
        # entries in the queue are created after schedule has elapsed.
        "publish_schedule": {"type": "datetime"},
        "publishing_action": {"type": "string"},
        "unique_name": {"type": "string", "nullable": True},
        "content_type": {"type": "string"},
        "headline": {"type": "string", "nullable": True},
        "transmit_started_at": {"type": "datetime"},
        "completed_at": {"type": "datetime"},
        "state": {"type": "string", "allowed": QueueState.values(), "nullable": False},
        "error_message": {"type": "string"},
        # to indicate the queue item is moved to legal
        # True is set after state of the item is success, cancelled or failed. For other state it is false
        "moved_to_legal": {"type": "boolean", "default": False},
        "retry_attempt": {"type": "integer", "default": 0},
        "next_retry_attempt_at": {"type": "datetime"},
        "ingest_provider": Resource.rel("ingest_providers", nullable=True),
        "associated_items": {"type": "list", "nullable": True},
        "priority": {
            "type": "boolean",
            "nullable": True,
        },
    }

    additional_lookup = {"url": r'regex("[\w,.:-]+")', "field": "item_id"}

    etag_ignore_fields = ["moved_to_legal"]
    datasource: Dict[str, Any] = {"default_sort": [("_id", -1), ("subscriber_id", 1), ("published_seq_num", -1)]}
    privileges = {"POST": "publish_queue", "PATCH": "publish_queue"}


class PublishQueueService(BaseService):
    def on_create(self, docs):
        subscriber_service = get_resource_service("subscribers")

        for doc in docs:
            self._set_queue_state(doc, {})
            doc["moved_to_legal"] = False

            if "published_seq_num" not in doc:
                subscriber = subscriber_service.find_one(req=None, _id=doc["subscriber_id"])
                doc["published_seq_num"] = subscriber_service.generate_sequence_number(subscriber)

    def on_updated(self, updates, original):
        if updates.get("state", "") != original.get("state", ""):
            self._set_queue_state(updates, original)
            push_notification(
                "publish_queue:update",
                queue_id=str(original["_id"]),
                completed_at=(updates.get("completed_at").isoformat() if updates.get("completed_at") else None),
                state=updates.get("state"),
                error_message=updates.get("error_message"),
            )

    def _set_queue_state(self, updates, original):
        destination = updates.get("destination", original.get("destination")) or {}
        updates["state"] = (
            QueueState.SUCCESS.value
            if destination.get("delivery_type") == "content_api"
            else updates.get("state") or QueueState.PENDING.value
        )

    def delete(self, lookup):
        # as encoded item is added manually to storage
        # we also need to remove it manually on delete
        cur = self.get_from_mongo(req=None, lookup=lookup).sort("_id", 1)
        for doc in cur:
            try:
                encoded_item_id = doc["encoded_item_id"]
            except KeyError:
                pass
            else:
                app.storage.delete(encoded_item_id)
        return super().delete(lookup)

    def delete_by_article_id(self, _id):
        lookup = {"item_id": _id}
        self.delete(lookup=lookup)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2019 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from ..service import ProdApiService


class PlanningService(ProdApiService):
    excluded_fields = {
        "_id",
        "item_class",
        "flags",
        "lock_user",
        "lock_time",
        "lock_session",
    } | ProdApiService.excluded_fields


class EventsService(ProdApiService):
    excluded_fields = {
        "_id",
        "lock_action",
        "lock_user",
        "lock_time",
        "lock_session",
    } | ProdApiService.excluded_fields


class AssignmentsService(ProdApiService):
    excluded_fields = {
        "lock_action",
        "lock_user",
        "lock_time",
    } | ProdApiService.excluded_fields


class EventsHistoryService(ProdApiService):
    excluded_fields = {
        "update._etag",
        "update._links",
        "update._status",
        "update._updated",
        "update._created",
        "update._id",
    } | ProdApiService.excluded_fields


class EventsFilesService(ProdApiService):
    pass



import codecs
import superdesk

from flask import current_app as app


class MigrateMediaCommand(superdesk.Command):
    """It will migrate files from Mongo GridFS into Amazon S3 storage."""

    option_list = [
        superdesk.Option("--limit", "-l", dest="limit", required=False, default=50, type=int),
        superdesk.Option("--skip", "-s", dest="skip", required=False, default=0, type=int),
    ]

    def run(self, limit, skip):
        mongo = app.media._storage[1]
        amazon = app.media._storage[0]

        files = mongo.fs().find(no_cursor_timeout=True).limit(limit).skip(skip)
        if not files.count():
            print("There are no files in mongo to be migrated.")
            return

        print("starting to migrate {} files".format(files.count()))
        migrated = 0

        for file in files:
            try:
                saved = amazon.put(
                    file.read(),
                    filename=file.filename,
                    content_type=file.content_type,
                    metadata=file.metadata,
                    _id=str(file._id),
                    ContentMD5=codecs.encode(codecs.decode(file.md5, "hex"), "base64").decode().strip(),
                )
                if saved:
                    mongo.delete(file._id)
                    migrated += 1
                    print(".", end="")
            except Exception as error:
                print("Error while migrating file {}: {}".format(file._id, error))

        print("done migrating {} files.".format(migrated))


superdesk.command("media:migrate", MigrateMediaCommand())



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from flask import g, current_app as app
from superdesk.resource import Resource
from superdesk.notification import push_notification
from superdesk.services import BaseService
from superdesk.errors import SuperdeskApiError
from .user_mentions import get_users, get_desks, get_mentions, notify_mentioned_users, notify_mentioned_desks
from flask_babel import _

comments_schema = {
    "text": {
        "type": "string",
        "minlength": 1,
        "maxlength": 500,
        "required": True,
    },
    "item": {"type": "string"},
    "user": Resource.rel("users", True),
    "mentioned_users": {"type": "dict"},
    "mentioned_desks": {"type": "dict"},
}


class CommentsResource(Resource):
    """Reusable implementation for comments."""

    schema = comments_schema
    resource_methods = ["GET", "POST", "DELETE"]
    datasource = {"default_sort": [("_created", -1)]}
    privileges = {"POST": "archive", "DELETE": "archive"}


def encode_keys(doc, field):
    """Encode field keys before storing.

    Mongo doesn't allow `.` in dict key.

    :param doc: doc
    :param field: doc field
    """
    if not doc.get(field):
        return
    encoded = {}
    for key, val in doc.get(field).items():
        encoded[key.replace(".", ";")] = val
    doc[field] = encoded


def decode_keys(doc, field):
    """Decode encoded field keys.

    :param doc: doc
    :param field: doc field
    """
    if not doc.get(field):
        return
    decoded = {}
    for key, val in doc.get(field).items():
        decoded[key.replace(";", ".")] = val
    doc[field] = decoded


class CommentsService(BaseService):
    notification_key = "comments"

    def on_create(self, docs):
        for doc in docs:
            sent_user = doc.get("user", None)
            user = g.user
            if sent_user and sent_user != str(user.get("_id")):
                message = _("Commenting on behalf of someone else is prohibited.")
                raise SuperdeskApiError.forbiddenError(message)
            doc["user"] = str(user.get("_id"))
            user_names, desk_names = get_mentions(doc.get("text"))
            doc["mentioned_users"] = get_users(user_names)
            doc["mentioned_desks"] = get_desks(desk_names)
            encode_keys(doc, "mentioned_users")
            encode_keys(doc, "mentioned_desks")

    def on_fetched(self, doc):
        for item in doc.get("_items", []):
            decode_keys(item, "mentioned_users")
            decode_keys(item, "mentioned_desks")

    def on_created(self, docs):
        for doc in docs:
            push_notification(self.notification_key, item=str(doc.get("item")))
            decode_keys(doc, "mentioned_users")
            decode_keys(doc, "mentioned_desks")

        notify_mentioned_users(docs, app.config.get("CLIENT_URL", "").rstrip("/"))
        notify_mentioned_desks(docs)

    def on_updated(self, updates, original):
        push_notification(self.notification_key, updated=1)

    def on_deleted(self, doc):
        push_notification(self.notification_key, deleted=1)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import re
import hashlib
from dateutil import parser

from superdesk.errors import IngestTwitterError
from superdesk.io.registry import register_feeding_service
from superdesk.io.feeding_services import FeedingService
from superdesk.metadata.utils import generate_guid
from superdesk.metadata.item import ITEM_TYPE, CONTENT_TYPE, GUID_TAG

import twitter


class TwitterFeedingService(FeedingService):
    """
    Feeding Service class which can read feed from multiple Twitter
    Screen Names and also hashtags
    """

    NAME = "twitter"

    label = "Twitter"

    fields = [
        {
            "id": "consumer_key",
            "type": "text",
            "label": "Twitter Consumer Key",
            "placeholder": "Twitter consumer_key",
            "required": True,
            "errors": {6100: "Twitter authentication failure"},
        },
        {
            "id": "consumer_secret",
            "type": "password",
            "label": "Twitter Consumer Secret",
            "placeholder": "Twitter consumer_secret",
            "required": True,
        },
        {
            "id": "screen_names",
            "type": "text",
            "label": "Twitter Screen Names",
            "placeholder": "Twitter screen_names",
            "required": True,
            "errors": {6200: "No Screen names specified"},
        },
    ]

    ERRORS = [
        IngestTwitterError.TwitterLoginError().get_error_description(),
        IngestTwitterError.TwitterNoScreenNamesError().get_error_description(),
        IngestTwitterError.TwitterRateLimitError().get_error_description(),
        IngestTwitterError.TwitterAPIGeneralError().get_error_description(),
    ]

    def _test(self, provider):
        self._update(provider, update=None, test=True)

    def _update(self, provider, update, test=False):
        config = provider.get("config", {})
        consumer_key = config.get("consumer_key", "")
        consumer_secret = config.get("consumer_secret", "")
        screen_names = config.get("screen_names", "")
        status_count = 100
        # how many statuses to get, 200 should be max

        new_items = []
        api = twitter.Api(consumer_key=consumer_key, consumer_secret=consumer_secret, application_only_auth=True)
        try:
            screen_names = screen_names.split(",")
        except Exception as ex:
            raise IngestTwitterError.TwitterNoScreenNamesError(ex, provider)

        if not screen_names:
            raise IngestTwitterError.TwitterNoScreenNamesError(provider)
        for screen_name in screen_names:
            screen_name = screen_name.replace(" ", "")
            try:
                # hashtag search
                if screen_name.startswith("#"):
                    statuses = api.GetSearch(screen_name.lstrip("#"), count=status_count)
                # user search
                else:
                    statuses = api.GetUserTimeline(screen_name=screen_name, count=status_count)
            except twitter.error.TwitterError as exc:
                # in some case python twitter error will return dict
                if type(exc.args[0]) == dict:
                    raise IngestTwitterError.TwitterAPIGeneralError(exc, provider)
                if exc.message[0].get("code") == 34:
                    # that page does not exist
                    continue
                elif exc.message[0].get("code") == 32:
                    # invalid credentials
                    raise IngestTwitterError.TwitterLoginError(exc, provider)
                elif exc.message[0].get("code") == 88:
                    # rate limit exceeded
                    raise IngestTwitterError.TwitterRateLimitError(exc, provider)
                else:
                    raise IngestTwitterError.TwitterAPIGeneralError(exc, provider)

            for status in statuses:
                d = parser.parse(status.created_at)
                guid_hash = hashlib.sha1(status.text.encode("utf8")).hexdigest()
                guid = generate_guid(type=GUID_TAG, id=guid_hash)
                headline = "%s: %s" % (status.user.screen_name, status.text)
                item = {}
                item["source"] = "twitter"
                item["extra"] = {"tweet_url": "https://twitter.com/%s/status/%s" % (status.user.screen_name, status.id)}
                item["headline"] = headline
                item["type"] = "text"
                item["guid"] = guid
                item["versioncreated"] = d
                item["firstcreated"] = d

                item["body_html"] = status.text
                # include URL on body
                urls = re.findall(
                    "http[s]?://(?:[a-zA-Z]|[0-9]|[$-_@.&+]|" "[!*\(\),]|(?:%[0-9a-fA-F][0-9a-fA-F]))+", status.text
                )
                if urls:
                    item["body_html"] += '<p><a href="%s"' ' target="_blank">%s</a></p>' % (urls[0], urls[0])

                # on hashtag search we don't want retweets
                if not (screen_name.startswith("#") and status.text.startswith("RT ")):
                    new_items.append(item)
                    if status.media:
                        image_urls = []
                        for media_item in status.media:
                            if media_item.type in ["photo", "animated", "video"]:
                                image_urls.append(media_item.media_url)
                        try:
                            # Eg can fail while fetching image
                            image_items = self._create_image_items(image_urls, item)
                            new_items.extend(image_items)
                            package_item = self._create_package(item, image_items)
                            new_items.append(package_item)
                        except Exception:
                            pass
        return [new_items]

    def _create_package(self, text_item, image_items):
        """
        Create a new content package from given content items.
        """
        package = {
            ITEM_TYPE: CONTENT_TYPE.COMPOSITE,
            "guid": generate_guid(type=GUID_TAG, id=text_item.get("guid") + "-package"),
            "versioncreated": text_item["versioncreated"],
            "firstcreated": text_item.get("firstcreated"),
            "headline": text_item.get("headline", ""),
            "groups": [
                {
                    "id": "root",
                    "role": "grpRole:NEP",
                    "refs": [{"idRef": "main"}],
                },
                {
                    "id": "main",
                    "role": "main",
                    "refs": [],
                },
            ],
        }

        item_references = package["groups"][1]["refs"]
        item_references.append({"residRef": text_item["guid"]})

        for image in image_items:
            item_references.append({"residRef": image["guid"]})

        return package

    def _create_image_items(self, image_links, text_item):
        image_items = []

        for image_url in image_links:
            guid_hash = hashlib.sha1(image_url.encode("utf8")).hexdigest()
            img_item = {
                "guid": generate_guid(type=GUID_TAG, id=text_item.get("guid") + guid_hash + "-image"),
                ITEM_TYPE: CONTENT_TYPE.PICTURE,
                "versioncreated": text_item.get("versioncreated"),
                "firstcreated": text_item.get("firstcreated"),
                "headline": text_item.get("headline", ""),
                "renditions": {"baseImage": {"href": image_url}},
            }
            image_items.append(img_item)

        return image_items


register_feeding_service(TwitterFeedingService)



import json
import time
import pytest
from requests.auth import _basic_auth_str
from flask import url_for

from superdesk.auth_server.scopes import Scope

from ..conftest import get_test_prodapi_app


RESOURCES = ("archive", "assignments", "contacts", "desks", "events", "planning", "users")

SCOPES = tuple(i.name for i in Scope)


def test_not_authenticated(prodapi_app, prodapi_client):
    """
    Send a not authenticated request:
        - make a request without a token
        - ensure that request is not authenticated by prod api

    :param prodapi_app: prod api app
    :type prodapi_app: eve.flaskapp.Eve
    :param prodapi_client: client for prod api
    :type prodapi_client: flask.testing.FlaskClient
    """

    with prodapi_app.test_request_context():
        for resource in RESOURCES:
            # we send a request without a token
            resp = prodapi_client.get(url_for("{}|resource".format(resource)))
            resp_data = json.loads(resp.data.decode("utf-8"))
            # we get a 401 response
            assert resp.status_code == 401
            assert resp_data["_status"] == "ERR"


@pytest.mark.parametrize("auth_server_registered_clients", [(("ARCHIVE_READ",),)], indirect=True)
def test_authenticated(superdesk_app, superdesk_client, auth_server_registered_clients):
    """
    Send an authenticated request:
        - register client for auth server
        - retrieve an acceess token from auth server by providing `id` and `password`
        - make a request with a token
        - ensure that request is authenticated by prod api

    :param superdesk_app: superdesk api app
    :type superdesk_app: eve.flaskapp.Eve
    :param superdesk_client: client for superdesk api
    :type superdesk_client: flask.testing.FlaskClient
    """

    client_id = auth_server_registered_clients[0]["client_id"]
    password = auth_server_registered_clients[0]["password"]

    # we send a client id and password to get an access token
    with superdesk_app.test_request_context():
        resp = superdesk_client.post(
            url_for("auth_server.issue_token"),
            data={"grant_type": "client_credentials"},
            headers={"Authorization": _basic_auth_str(client_id, password)},
        )

    # we get an access token
    resp_data = json.loads(resp.data.decode("utf-8"))
    assert resp.status_code == 200
    assert resp_data["token_type"] == "Bearer"
    assert "access_token" in resp_data

    # we create a prodapi flask app and client
    prodapi_app = get_test_prodapi_app()
    prodapi_client = prodapi_app.test_client()

    # we send a request with an auth token
    token = resp_data["access_token"]
    with prodapi_app.test_request_context():
        for resource in ("archive",):
            # we send a request with a token
            resp = prodapi_client.get(
                url_for("{}|resource".format(resource)), headers={"Authorization": "Bearer {}".format(token)}
            )

            # we get a 200 response
            assert resp.status_code == 200


@pytest.mark.parametrize("auth_server_registered_clients", [(("ARCHIVE_READ",),)], indirect=True)
def test_bad_shared_key(superdesk_app, superdesk_client, auth_server_registered_clients):
    """
    Send a request with a token which is signed and verified using different secrets:
        - register client for auth server
        - retrieve an acceess token from auth server by providing `id` and `password`
        - make a request with a token
        - ensure that request is NOT authenticated by prod api, because of different shared secret

    :param superdesk_app: superdesk api app
    :type superdesk_app: eve.flaskapp.Eve
    :param superdesk_client: client for superdesk api
    :type superdesk_client: flask.testing.FlaskClient
    """

    client_id = auth_server_registered_clients[0]["client_id"]
    password = auth_server_registered_clients[0]["password"]

    # we send a client id and password to get an access token
    with superdesk_app.test_request_context():
        resp = superdesk_client.post(
            url_for("auth_server.issue_token"),
            data={"grant_type": "client_credentials"},
            headers={"Authorization": _basic_auth_str(client_id, password)},
        )

    # we get an access token
    resp_data = json.loads(resp.data.decode("utf-8"))
    token = resp_data["access_token"]

    # we create a prodapi flask app and client
    prodapi_app = get_test_prodapi_app()
    prodapi_client = prodapi_app.test_client()

    # we change `AUTH_SERVER_SHARED_SECRET`
    prodapi_app.config["AUTH_SERVER_SHARED_SECRET"] = "7fZOf0VI9T70vU5uNlKLrc5GlabxVgl6"

    # we send a request with an auth token
    with prodapi_app.test_request_context():
        for resource in ("archive",):
            # we send a request with a token
            resp = prodapi_client.get(
                url_for("{}|resource".format(resource)), headers={"Authorization": "Bearer {}".format(token)}
            )
            resp_data = json.loads(resp.data.decode("utf-8"))

    # we get a 401 response
    assert resp.status_code == 401
    assert resp_data == {"_status": "ERR", "_error": {"code": 401, "message": "Please provide proper credentials"}}


@pytest.mark.parametrize("issued_tokens", [(SCOPES, tuple())], indirect=True)
def test_scopes(issued_tokens, prodapi_app, prodapi_client):
    """
    Send a request with 'all scopes' token:
        - make a requests to all endpoints with a token
        - ensure that all requests are authorized by prod api

    Send a request with 'empty scopes' token:
        - make a requests to all endpoints with a token
        - ensure that all requests are NOT authorized by prod api due to empty scopes

    :param issued_tokens: list of tokens provided by `issued_tokens` fixture
    :type issued_tokens: list
    :param prodapi_app: prod api app
    :type prodapi_app: eve.flaskapp.Eve
    :param prodapi_client: client for prod api
    :type prodapi_client: flask.testing.FlaskClient
    """

    access_token_all_scopes = issued_tokens[0]["access_token"]
    access_token_no_scopes = issued_tokens[1]["access_token"]

    with prodapi_app.test_request_context():
        for resource in RESOURCES:
            # we send a requests with 'empty scopes' token
            resp = prodapi_client.get(
                url_for("{}|resource".format(resource)),
                headers={"Authorization": "Bearer {}".format(access_token_no_scopes)},
            )
            resp_data = json.loads(resp.data.decode("utf-8"))
            # we get a 403 response
            assert resp.status_code == 403
            assert resp_data == {"_status": "ERR", "_error": {"code": 403, "message": "Invalid scope"}}
            # we send a requests with 'all scopes' token
            resp = prodapi_client.get(
                url_for("{}|resource".format(resource)),
                headers={"Authorization": "Bearer {}".format(access_token_all_scopes)},
            )
            # we get a 200 response
            assert resp.status_code == 200


@pytest.mark.parametrize("issued_tokens", [(("DESKS_READ",),)], indirect=True)
def test_scopes_partial(issued_tokens, prodapi_app):
    """
    Send a request with DESKS_READ scope token to `desks` and `items`:
        - make requests to `desks` and `items` endpoints
        - ensure that only request to `desk` is authorized

    :param prodapi_app: prod api app
    :type prodapi_app: eve.flaskapp.Eve
    :param issued_tokens: list of tokens provided by `issued_tokens` fixture
    :type issued_tokens: list
    """

    access_token = issued_tokens[0]["access_token"]
    prodapi_client = prodapi_app.test_client()

    with prodapi_app.test_request_context():
        # we send a request with 'DESKS_READ scope' token to desks endpoint
        resp = prodapi_client.get(
            url_for("desks|resource"), headers={"Authorization": "Bearer {}".format(access_token)}
        )
        # we get a 200 response
        assert resp.status_code == 200

        # we send a request with 'DESKS_READ scope' token to items endpoint
        resp = prodapi_client.get(
            url_for("archive|resource"), headers={"Authorization": "Bearer {}".format(access_token)}
        )
        resp_data = json.loads(resp.data.decode("utf-8"))
        # we get a 403 response
        assert resp.status_code == 403
        assert resp_data == {"_status": "ERR", "_error": {"code": 403, "message": "Invalid scope"}}


@pytest.mark.parametrize("auth_server_registered_clients", [(("ARCHIVE_READ",),)], indirect=True)
@pytest.mark.parametrize(
    "superdesk_app",
    [
        {"AUTH_SERVER_EXPIRATION_DELAY": 1},
    ],
    indirect=True,
)
def test_token_expired(superdesk_app, superdesk_client, auth_server_registered_clients):
    """
    Send a request with an expired token:
        - register client for auth server
        - retrieve an acceess token from auth server by providing `id` and `password`
        - wait until token expires
        - make a request with an expired token
        - ensure that request is NOT authenticated by prod api

    :param superdesk_app: superdesk api app
    :type superdesk_app: eve.flaskapp.Eve
    :param superdesk_client: client for superdesk api
    :type superdesk_client: flask.testing.FlaskClient
    """

    client_id = auth_server_registered_clients[0]["client_id"]
    password = auth_server_registered_clients[0]["password"]
    expiration_delay = superdesk_app.config["AUTH_SERVER_EXPIRATION_DELAY"]

    # we send a client id and password to get an access token
    with superdesk_app.test_request_context():
        resp = superdesk_client.post(
            url_for("auth_server.issue_token"),
            data={"grant_type": "client_credentials"},
            headers={"Authorization": _basic_auth_str(client_id, password)},
        )

    # we get an access token
    assert resp.status_code == 200
    resp_data = json.loads(resp.data.decode("utf-8"))
    token = resp_data["access_token"]

    # wait until token expire
    time.sleep(expiration_delay + 1)

    # we create a prodapi flask app and client
    prodapi_app = get_test_prodapi_app()
    prodapi_client = prodapi_app.test_client()

    # we send a request with an expired auth token
    with prodapi_app.test_request_context():
        for resource in ("archive",):
            # we send a request with a token
            resp = prodapi_client.get(
                url_for("{}|resource".format(resource)), headers={"Authorization": "Bearer {}".format(token)}
            )

    # we get a 401 response
    assert resp.status_code == 401



"""Superdesk places module.

Provides places autocomplete and search.
"""

import superdesk

from .places_autocomplete import PlacesAutocompleteResource, PlacesAutocompleteService


def init_app(app):
    if app.config.get("GEONAMES_USERNAME"):
        superdesk.register_resource(
            "places_autocomplete", PlacesAutocompleteResource, PlacesAutocompleteService, _app=app
        )



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from superdesk.resource import Resource


class ProfilingResource(Resource):
    """
    Profiling schema
    """

    schema = {
        "name": {"type": "string", "required": True, "unique": True},
        "profiling_data": {"type": "list", "required": True},
    }
    item_url = 'regex("[\w,.:-]+")'
    item_methods = ["GET"]
    resource_methods = ["GET", "POST", "DELETE"]
    privileges = {"POST": "profiling", "DELETE": "profiling"}



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013-2019 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import os
import copy
from io import BytesIO
from unittest.mock import MagicMock

import magic
import requests_mock
from werkzeug.datastructures import FileStorage

import superdesk
from superdesk import config
from superdesk.errors import SuperdeskApiError
from superdesk.tests import TestCase


class Req(dict):
    args = {}


video_info = {
    "_id": "video_id",
}


class VideoEditTestCase(TestCase):
    project_data = {
        "_id": "video_id",
        "metadata": {
            "codec_name": "h264",
            "width": 1280,
            "height": 720,
            "duration": 10,
        },
        "url": "video_url",
        "mime_type": "video/mp4",
        "version": 1,
    }

    def setUp(self):
        self.video_edit = superdesk.get_resource_service("video_edit")
        self.app.config["VIDEO_SERVER_ENABLED"] = "true"
        self.app.config["VIDEO_SERVER_URL"] = "http://localhost"
        with requests_mock.mock() as mock:
            dirname = os.path.dirname(os.path.realpath(__file__))
            video_file_path = os.path.normpath(os.path.join(dirname, "fixtures", "envy_dog.mp4"))

            with open(video_file_path, "rb") as f:
                doc = {"media": FileStorage(BytesIO(f.read()), "video.mp4", content_type="video/mp4")}

            mock.post("http://localhost/projects/", json=self.project_data)
            mock.get("http://localhost/projects/video_id/thumbnails?type=timeline&amount=60", json={"processing": True})
            archive_service = superdesk.get_resource_service("archive")
            magic.from_buffer = MagicMock()
            magic.from_buffer.return_value = "video/mp4"
            self.item = archive_service.find_one(req=None, _id=archive_service.post([doc])[0])

    def test_get_video(self):
        with requests_mock.mock() as mock:
            mock.get("http://localhost/projects/video_id", json=video_info)
            res = self.video_edit.find_one(Req(), _id=self.item["_id"])
            self.assertEqual(res["project"]["_id"], video_info["_id"])

    def test_upload_video(self):
        self.assertEqual(
            self.item["renditions"],
            {"original": {"href": "video_url", "mimetype": "video/mp4", "version": 1, "video_editor_id": "video_id"}},
        )

    def test_missing_video_id(self):
        doc = {"item": {config.ID_FIELD: "123", "renditions": {"original": {}}}}
        with self.assertRaises(SuperdeskApiError) as ex:
            self.video_edit.create([doc])
        self.assertEqual(ex.exception.message, '"video_editor_id" is required')

    def test_missing_action(self):
        doc = {"item": {config.ID_FIELD: "123", "renditions": {"original": {"video_editor_id": "video_id"}}}}
        with self.assertRaises(SuperdeskApiError) as ex:
            self.video_edit.create([doc])
        self.assertEqual(ex.exception.message, '"capture" or "edit" is required')

    def test_edit_video(self):
        project_data = copy.deepcopy(self.project_data)
        doc = {
            "item": {
                config.ID_FIELD: self.item[config.ID_FIELD],
                "renditions": self.item["renditions"],
            },
            "edit": {"crop": "0,0,200,500", "rotate": -90, "trim": "5,15"},
        }
        with requests_mock.mock() as mock:
            project_data["version"] = 2
            mock.get("http://localhost/projects/video_id", json=project_data)
            mock.post("http://localhost/projects/video_id/duplicate", json=project_data)
            mock.put("http://localhost/projects/video_id", json={"processing": True})
            item = self.video_edit.find_one(req=None, _id=self.video_edit.create([doc])[0])
            self.assertEqual(
                item["renditions"],
                {
                    "original": {
                        "href": "video_url",
                        "mimetype": "video/mp4",
                        "version": 3,
                        "video_editor_id": "video_id",
                    }
                },
            )

    def test_capture_thumbnail(self):
        doc = {
            "item": {
                config.ID_FIELD: self.item[config.ID_FIELD],
                "renditions": self.item["renditions"],
            },
            "capture": {"crop": "0,0,200,500", "rotate": -90, "trim": "5,10"},
        }
        project_data = copy.deepcopy(self.project_data)
        project_data.setdefault("thumbnails", {})["preview"] = {
            "mimetype": "image/png",
            "url": "http://localhost/projects/video_id/raw/thumbnails/preview",
        }
        with requests_mock.mock() as mock:
            mock.get("http://localhost/projects/video_id", json=project_data)
            mock.post("http://localhost/projects/video_id", json={"processing": True})
            mock.get(
                "http://localhost/projects/video_id/thumbnails?type=preview&crop=0,0,200,500&rotate=-90",
                json=project_data,
            )
            item = self.video_edit.find_one(req=None, _id=self.video_edit.create([doc])[0])
            self.assertEqual(
                item["renditions"],
                {
                    "original": {
                        "href": "video_url",
                        "mimetype": "video/mp4",
                        "version": 1,
                        "video_editor_id": "video_id",
                    },
                    "viewImage": {
                        "mimetype": "image/png",
                        "href": "http://localhost/projects/video_id/raw/thumbnails/preview",
                    },
                },
            )

    def test_upload_thumbnail(self):
        thumbnail = {"mimetype": "image/jpeg", "url": "http://localhost/projects/video_id/raw/thumbnails/preview"}
        with requests_mock.mock() as mock:
            mock.post("http://localhost/projects/video_id/thumbnails", json=thumbnail)
            req = {"file": FileStorage(BytesIO(b"abcdef"), "image.jpeg")}
            res = self.video_edit.on_replace(req, {"project": {"_id": "video_id"}})
            self.assertEqual(
                res["renditions"]["viewImage"]["href"], "http://localhost/projects/video_id/raw/thumbnails/preview"
            )
            self.assertEqual(res["renditions"]["viewImage"]["mimetype"], "image/jpeg")

    def test_capture_timeline(self):
        with requests_mock.mock() as mock:
            mock.get(
                "http://localhost/projects/video_id/thumbnails?type=timeline&amount=60",
                json={"processing": True},
                status_code=202,
            )
            req = Req()
            setattr(req, "args", {"action": "timeline"})
            res = self.video_edit.find_one(req, _id=self.item["_id"])
            self.assertEqual(res[config.ID_FIELD], video_info["_id"])
            self.assertTrue(res["processing"])



from flask import current_app, json

from superdesk.publish import register_transmitter
from superdesk.publish.publish_service import PublishService
from superdesk.text_checkers.ai.imatrics import IMatrics


class IMatricsTransmitter(PublishService):
    def _transmit(self, queue_item, subscriber):
        imatrics = IMatrics(current_app)
        item = json.loads(queue_item["formatted_item"])
        imatrics.publish(item)


register_transmitter("imatrics", IMatricsTransmitter(), [])



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


import os
from superdesk.tests import TestCase

from superdesk.io.feed_parsers.ap_anpa import AP_ANPAFeedParser


def fixture(filename):
    dirname = os.path.dirname(os.path.realpath(__file__))
    return os.path.normpath(os.path.join(dirname, "../fixtures", filename))


class ANPATestCase(TestCase):

    parser = AP_ANPAFeedParser()

    def setUp(self):
        with self.app.app_context():
            vocab = [
                {"_id": "categories", "items": [{"is_active": True, "name": "Domestic Sport", "qcode": "s"}]},
                {
                    "_id": "ap_category_map",
                    "items": [
                        {"is_active": True, "ap_code": "e", "category_code": "e"},
                        {"is_active": True, "ap_code": "f", "category_code": "f"},
                        {"is_active": True, "ap_code": "j", "category_code": "l"},
                        {"is_active": True, "ap_code": "l", "category_code": "e"},
                        {"is_active": True, "ap_code": "q", "category_code": "s"},
                        {"is_active": True, "ap_code": "r", "category_code": "v"},
                        {"is_active": True, "ap_code": "s", "category_code": "s"},
                        {"is_active": True, "ap_code": "z", "category_code": "s"},
                        {"is_active": True, "ap_code": "default", "category_code": "i"},
                    ],
                },
            ]
            self.app.data.insert("vocabularies", vocab)

    def open(self, filename):
        provider = {"name": "Test"}
        return self.parser.parse(fixture(filename), provider)

    def test_ed_note(self):
        item = self.open("anpa-2.tst")
        self.assertEqual(
            "This is part of an Associated Press investigation into the hidden costs of green energy.", item["ednote"]
        )
        self.assertEqual("1stLd-Writethru", item["anpa_take_key"])
        self.assertEqual("i", item["anpa_category"][0]["qcode"])

    def test_subject_expansion(self):
        item = self.open("ap_anpa-1.tst")
        self.assertEqual(item["subject"][0]["qcode"], "15008000")
        self.assertEqual(item["dateline"]["text"], "ATLANTA, Feb 19 AP -")
        self.assertEqual(item["anpa_category"][0]["qcode"], "s")

    def test_table_story(self):
        item = self.open("ap_anpa-2.tst")
        self.assertEqual(item["slugline"], "BBO--BaseballExpanded")
        self.assertEqual(item["format"], "preserved")
        self.assertGreater(item["body_html"].find("%08Baltimore;23;13;.639;_;_;8-2;L-1;16-6;7-7"), 0)

    def test_unknown_category_defaults_to_i(self):
        item = self.open("ap_anpa-3.tst")
        self.assertEqual(item["anpa_category"][0]["qcode"], "i")

    def test_ed_note_with_parentesis(self):
        item = self.open("ap_anpa-4.tst")
        self.assertEqual(
            item["ednote"],
            "(Minor edits. A longer version of this story is available. "
            "With AP Photos. With BC-AS--China-Lawyer Trial.)",
        )

    def test_alert(self):
        item = self.open("ap_anpa-5.tst")
        self.assertTrue(
            item["headline"],
            "Hawaii files court challenge to Trump administration"
            "s definition of "
            "close U.S. relationship needed to avoid travel ban.",
        )

    def test_no_word_count(self):
        item = self.open("ap_anpa-6.tst")
        self.assertEqual(item["slugline"], "BBO--WildCardGlance")
        self.assertEqual(item["anpa_category"], [{"qcode": "s"}])

    def test_number_in_slugline(self):
        item = self.open("ap_anpa-7.tst")
        self.assertEqual(item["slugline"], "10ThingstoKnow-Today")



# -*- coding: utf-8; -*-
# This file is part of Superdesk.
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license
#
# Author  : administrator
# Creation: 2020-11-04 00:16

import pymongo.errors

from superdesk.commands.data_updates import BaseDataUpdate


class DataUpdate(BaseDataUpdate):

    resource = "content_templates"

    def forwards(self, mongodb_collection, mongodb_database):
        try:
            mongodb_collection.drop_index("user_1_template_name_1")
        except pymongo.errors.OperationFailure:
            pass

    def backwards(self, mongodb_collection, mongodb_database):
        pass



import unittest
import requests_mock

from apps.vidible.vidible import get_vidible_metadata


VIDIBLE_TEXT = """
(function () {
    _(
        {"status":{"code":"OK"},"jsVersion":"17.16.1039","id":"56bb474de4b0568f54a23ed7","wlcid":"538612f0e4b00fbb8e898655","bid":{"id":"56bb4724e4b05485f65670bd","videos":[{"videoId":"56bb4688e4b0b6448ed479dd","name":"Watch CCTV of incident in Romford McDonald's","videoUrls":["http://videos.vidible.tv/prod/2016-02/10/56bb4688e4b0b6448ed479dd_640x360_v1.mp4?oxdeb1EtQ4XC5ta_uM6pLRuyXdX_A09cVu_Uw9iwA_RUnZxrbI4ZKDeWOI4gLiPm","http://videos.vidible.tv/prod/2016-02/10/56bb4688e4b0b6448ed479dd_640x360_v1.ogg?Eqi5v8YcCAlqziP1mA8VPrXfta1eyHkhT46B7tdG5yzmOkoalIkvBMwy-52CWvAS"],"thumbnailId":"56bb46b4e4b0ba78d5744c69","thumbnail":"http://cdn.vidible.tv/prod/2016-02/10/56bb4688e4b0b6448ed479dd_60x60_A_v1.jpg","fullsizeThumbnail":"http://cdn.vidible.tv/prod/2016-02/10/56bb4688e4b0b6448ed479dd_1024x576_A_v1.jpg","subtitles":[],"captions":{},"metadata":{"duration":24000,"videoType":"VOD","commonRating":{"value":"G","descriptors":[],"minAge":0}},"videoSourceType":"FILE","iabCategories":[],"studioName":"Press Association","cuePoints":[],"cs":{"priority":false}}]},"playerTemplate":{"initialization":"click","mobileInit":"CLICK","mobileAutoplayWifiOnly":true,"fullscreenOnUnmute":false,"previewPoster":"IMAGE","sound":"normal","initialVolume":0.5,"videosToPlay":1,"videosToRequest":1,"shuffleVideos":false,"prerollFrequency":0,"backgroundSkinLocation":{"x":0,"y":0,"w":0,"h":0},"controlsSkin":"http://cdn.vidible.tv/prod/player/swf/17.06.342/ControlsAOL_5.swf","controlsSkinLocation":{"x":0,"y":0,"w":640,"h":360},"videoLocation":{"x":0,"y":0,"w":640,"h":360},"afterVideosUI":"next","scrubBarSkin":"http://cdn.vidible.tv/prod/2013-03/10/511e8160e4b0bf40bd0340a7_v2.swf","coveringsSkinLocation":{"x":0,"y":0,"w":640,"h":360},"surroundSkinLocation":{"x":0,"y":0,"w":0,"h":0},"playerWidth":640,"playerHeight":360,"controlsAutoPosition":true,"controlsChromeless":true,"controlsBackgroundAlpha":"1.0","controlsConfig":{"colorSchema":{"mainColor":{"backgroundAlpha":1.0}}},"nielsenSiteCampaign":"nlsn31325","nielsenChannelCampaign":"cmp185270","backgroundFill":true,"backgroundColor":0,"surround3DBevelShadowColor":16777215,"surround3DBevelHighlightColor":16777215,"surroundInnerRadius":0,"surroundCornerRadius":0,"surroundHole":false,"surround3D":false,"surround3DBevelSize":0,"surround3DBevelStrength":0.0,"publisherPayout":"None","publisherAmount":0.0,"metaData":{},"showLogo":false,"autoplayInView":0,"vrRenderer":"O2","isResponsive":false,"csid":"54ef07a8e4b020e2f828c02c","HLSExtra":"http://cdn.vidible.tv/prod/player/swf/17.06.342/hls-plugin.swf","IMAExtra":"http://cdn.vidible.tv/prod/player/swf/17.06.342/ima-ad-module.swf","YuMeExtra":"http://cdn.vidible.tv/prod/player/swf/17.06.342/yume-ad-module.swf","FreeWheelExtra":"http://cdn.vidible.tv/prod/player/swf/17.06.342/free-wheel-module.swf","VASTExtra":"http://cdn.vidible.tv/prod/player/swf/17.06.342/vast-ad-engine.swf","PlayerAdSystem":"http://cdn.vidible.tv/prod/player/swf/17.06.342/vidible-ad-server.swf","UIExtra":"http://cdn.vidible.tv/prod/player/swf/17.06.342/controls-sticky.swf","AgeGateExtra":"http://cdn.vidible.tv/prod/player/swf/17.06.342/age-gate.swf","SubtitlesExtra":"http://cdn.vidible.tv/prod/player/swf/17.06.342/subtitles.swf","ClickExtra":"http://cdn.vidible.tv/prod/player/swf/17.06.342/click-throughs.swf"},"timestamp":"20170623114355688","adSettings":{"podSize":1,"prerollInterleave":1,"strictSiteCheckForAds":false,"blockContentOnAdBlocker":false,"domainOptimisation":true,"adStrategy":"ADSET_BASED","useSsai":false,"companions":[],"aeg":[],"asids":[]},"playerWidget":{"playerType":"SMART","url":"http://cdn.vidible.tv/prod/player/swf/17.06.342/player-vast.swf","adOnly":false,"isAol":false},"geo":{"country":"cze","region":"10","zipCode":"100 00","areaCode":"0","connSpeed":"broadband"},"brandedContent":false,"sid":"97c459e9-d2da-495f-a537-edd01812dd3c","bsf":{"bs":{},"f":{"o2":0.0}}}
    );
    function _(json) {
    }
})()
"""  # noqa

SEARCH_TEXT = """
[{"id":"56bb4688e4b0b6448ed479dd","title":"Watch CCTV of incident in Romford McDonald's","description":"Detectives from Havering Borough investigating an attempted robbery at a McDonalds restaurant have issued CCTV footage of three men they wish to speak to in connection with the incident.","thumbnail":"http://img.vidible.tv/prod/2016-02/10/56bb4688e4b0b6448ed479dd_o_A_v1.jpg","url":"http://delivery.vidible.tv/video/redirect/56bb4688e4b0b6448ed479dd?bcid=538612f0e4b00fbb8e898655&w=640&h=360","size":2004079,"creationDate":1455113864099,"company":"Press Association","duration":24000,"width":640,"height":360,"mimeType":"video/ogg","modificationDate":1462876434275,"media":{},"plays":0,"companyId":"538612f0e4b00fbb8e898655","geoRestriction":{"whiteList":[],"blackList":[],"blackListMode":false},"deviceRestriction":{"whiteList":[],"blackListMode":false},"shortDescription":"Detectives from Havering Borough investigating an attempted robbery at a McDonalds restaurant have issued CCTV footage of three men they wish to speak to in connection with the incident."}]
"""  # noqa


class VidibleTestCase(unittest.TestCase):
    def test_get_vidible_metadata(self):
        pid = "56bb474de4b0568f54a23ed7"
        vid = "56bb4688e4b0b6448ed479dd"
        bcid = "538612f0e4b00fbb8e898655"
        with requests_mock.mock() as mock:
            mock.get(
                "http://delivery.vidible.tv/jsonp/pid={pid}/{bcid}.js".format(pid=pid, bcid=bcid), text=VIDIBLE_TEXT
            )
            mock.get(
                "http://api.vidible.tv/search?bcid={bcid}&query={video_id}".format(video_id=vid, bcid=bcid),
                text=SEARCH_TEXT,
            )
            meta = get_vidible_metadata(bcid=bcid, pid=pid)
        assert type(meta) is dict
        assert type(meta["height"]) is int
        assert type(meta["width"]) is int
        assert meta["mimeType"]
        assert meta["url"]
        assert meta["thumbnail"]



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


import os
from superdesk.tests import TestCase
from superdesk.io.feed_parsers.ninjs import NINJSFeedParser


class NINJSTestCase(TestCase):
    vocab = [{"_id": "genre", "items": [{"name": "Current"}]}]

    def setUp(self):
        with self.app.app_context():
            self.app.data.insert("vocabularies", self.vocab)
        dirname = os.path.dirname(os.path.realpath(__file__))
        fixture = os.path.normpath(os.path.join(dirname, "../fixtures", self.filename))
        provider = {"name": "Test"}
        self.items = NINJSFeedParser().parse(fixture, provider)


class SimpleTestCase(NINJSTestCase):

    filename = "ninjs1.json"

    def test_headline(self):
        self.assertEqual(self.items[0].get("headline"), "headline")
        self.assertEqual(self.items[0].get("description_text"), "abstract")
        self.assertEqual(1, len(self.items[0].get("authors")), "authors")
        self.assertEqual(
            self.items[0]["authors"][0],
            {
                "name": "John",
                "role": "writer",
                "avatar_url": "http://example.com",
                "biography": "bio",
            },
        )
        self.assertNotIn("source", self.items[0])
        self.assertEqual(self.items[0]["original_source"], "AAP")
        self.assertEqual("2017-08-24T04:38:34+00:00", self.items[0]["versioncreated"].isoformat())


class AssociatedTestCase(NINJSTestCase):

    filename = "ninjs2.json"

    def test_parsed_items(self):
        # The picture
        self.assertEqual(self.items[0].get("type"), "picture")
        self.assertEqual(self.items[0].get("headline"), "Financial Markets")
        self.assertEqual(self.items[0].get("alt_text"), "Oil markets something")
        self.assertEqual(
            self.items[0].get("description_text"),
            "Oil prices edged up on Monday, supported by the first fall in US drilling activity in month.",
        )
        # The text item
        self.assertEqual(self.items[1].get("type"), "text")
        self.assertEqual(self.items[1].get("headline"), "Oil prices edge up on first drop in US drilling in months")
        self.assertEqual(
            self.items[1].get("abstract"),
            "Oil prices edged up on Monday, supported by the first fall in US drilling activity in month.",
        )
        # The associated picture in the text item
        self.assertEqual(self.items[1].get("associations").get("featuremedia").get("type"), "picture")
        self.assertEqual(self.items[1].get("associations").get("featuremedia").get("alt_text"), "Oil markets something")
        self.assertEqual(
            self.items[1].get("associations").get("featuremedia").get("description_text"),
            "Oil prices edged up on Monday, supported by the first fall in US drilling activity in month.",
        )


class PictureTestCase(NINJSTestCase):

    filename = "ninjs3.json"

    def test_headline(self):
        self.assertEqual(self.items[0].get("headline"), "German Air Force Museum")
        self.assertEqual(self.items[0].get("type"), "picture")


class IPTCSimpleTextTestCase(NINJSTestCase):
    filename = "ninjsExSimpleText1.json"

    def test_simple(self):
        self.assertEqual(1, len(self.items))
        item = self.items[0]
        self.assertEqual("urn:ninjs.example.com:newsitems:20130709simp123", item["guid"])
        self.assertEqual("2013-07-09T10:37:00+00:00", item["versioncreated"].isoformat())
        self.assertIn("GROSSETO", item["body_html"])


class IPTCMediumTextTestCase(NINJSTestCase):
    filename = "ninjsExMediumText1.json"

    def test_medium(self):
        self.assertEqual(1, len(self.items))
        self.assertEqual("text-only", self.items[0]["profile"])
        self.assertEqual("en", self.items[0]["language"])



import unittest
from datetime import datetime

from superdesk.io.subjectcodes import SubjectIndex
from superdesk.io.iptc import init_app


class SubjectcodeTestCase(unittest.TestCase):
    def setUp(self):
        self.subjects = SubjectIndex()

    def test_subjectcodes_empty(self):
        self.assertEqual([], self.subjects.get_items())

    def test_subjectcodes_register(self):
        self.subjects.register(
            {
                "01000000": "arts",
            },
            last_modified=datetime(2015, 10, 10),
        )

        self.assertEqual(1, len(self.subjects.get_items()))

        self.subjects.register(
            {
                "01001000": "archeology",
            },
            last_modified=datetime(2015, 1, 1),
        )

        self.assertEqual(2, len(self.subjects.get_items()))
        self.assertEqual(datetime(2015, 10, 10), self.subjects.last_modified)
        self.assertEqual("01000000", self.subjects.get_items()[1]["parent"])

        self.subjects.register({"subj:test": "test"})

        self.assertEqual(3, len(self.subjects.get_items()))
        self.assertEqual(None, self.subjects.get_items()[2]["parent"])

    def test_iptc_init(self):
        init_app(self)
        self.assertEqual(1404, len(self.subjects.get_items()))



import superdesk
from superdesk.metadata.item import ITEM_TYPE, CONTENT_TYPE, CONTENT_STATE, ITEM_STATE, get_schema
from flask import render_template, render_template_string
from superdesk.errors import SuperdeskApiError
from eve.utils import config
from flask import current_app as app
from .service import get_highlight_name
from apps.archive.common import ITEM_EXPORT_HIGHLIGHT, ITEM_CREATE_HIGHLIGHT

PACKAGE_FIELDS = {
    "type",
    "state",
    "groups",
    "unique_name",
    "pubstatus",
    "origina_creator",
    "flags",
    "guid",
    "schedule_settings",
    "expiry",
    "format",
    "lock_time",
    "lock_user",
    "lock_session",
    config.ID_FIELD,
    config.LAST_UPDATED,
    config.DATE_CREATED,
    config.ETAG,
    "version",
    "_current_version",
    "version_creator",
    "operation",
    "unique_id",
    "version_created",
    "fields_meta",
}


def get_template(highlightId):
    """Return the string template associated with highlightId or none"""
    if not highlightId:
        return None
    highlightService = superdesk.get_resource_service("highlights")
    highlight = highlightService.find_one(req=None, _id=highlightId)
    if not highlight or not highlight.get("template"):
        return None

    templateService = superdesk.get_resource_service("content_templates")
    template = templateService.find_one(req=None, _id=highlight.get("template"))
    return template


class GenerateHighlightsService(superdesk.Service):
    def create(self, docs, **kwargs):
        """Generate highlights text item for given package.

        If doc.preview is True it won't save the item, only return.
        """
        service = superdesk.get_resource_service("archive")
        for doc in docs:
            preview = doc.get("preview", False)
            package = service.find_one(req=None, _id=doc["package"])
            if not package:
                superdesk.abort(404)
            export = doc.get("export")
            template = get_template(package.get("highlight"))
            stringTemplate = None
            if template and template.get("data") and template["data"].get("body_html"):
                stringTemplate = template["data"]["body_html"]

            doc.clear()
            doc[ITEM_TYPE] = CONTENT_TYPE.TEXT
            doc["family_id"] = package.get("guid")
            doc[ITEM_STATE] = CONTENT_STATE.SUBMITTED
            doc[config.VERSION] = 1

            for field in package:
                if field not in PACKAGE_FIELDS:
                    doc[field] = package[field]

            items = []
            for group in package.get("groups", []):
                for ref in group.get("refs", []):
                    if "residRef" in ref:
                        item = service.find_one(req=None, _id=ref.get("residRef"))
                        if item:
                            if not (export or preview) and (
                                item.get("lock_session") or item.get("state") != "published"
                            ):
                                message = "Locked or not published items in highlight list."
                                raise SuperdeskApiError.forbiddenError(message)

                            items.append(item)
                            if not preview:
                                app.on_archive_item_updated(
                                    {
                                        "highlight_id": package.get("highlight"),
                                        "highlight_name": get_highlight_name(package.get("highlight")),
                                    },
                                    item,
                                    ITEM_EXPORT_HIGHLIGHT,
                                )

            if stringTemplate:
                doc["body_html"] = render_template_string(stringTemplate, package=package, items=items)
            else:
                doc["body_html"] = render_template("default_highlight_template.txt", package=package, items=items)
        if preview:
            return ["" for doc in docs]
        else:
            ids = service.post(docs, **kwargs)
            for id in ids:
                app.on_archive_item_updated(
                    {
                        "highlight_id": package.get("highlight"),
                        "highlight_name": get_highlight_name(package.get("highlight")),
                    },
                    {"_id": id},
                    ITEM_CREATE_HIGHLIGHT,
                )
            return ids


class GenerateHighlightsResource(superdesk.Resource):
    """Generate highlights item for given package."""

    schema = {
        "package": {
            # not setting relation here, we will fetch it anyhow
            "type": "string",
            "required": True,
        },
        "preview": {
            "type": "boolean",
            "default": False,
        },
        "export": {
            "type": "boolean",
            "default": False,
        },
        "type": {
            "type": "string",
            "readonly": True,
        },
    }
    schema.update(get_schema(versioning=True))

    resource_methods = ["POST"]
    item_methods = []
    privileges = {"POST": "highlights"}



# Copyright 2020 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from superdesk.resource import Resource
from superdesk.services import BaseService
from superdesk.errors import SuperdeskApiError
from .. import tools
from .base import registered_ai_services, AIServiceBase
import superdesk

AUTO_IMPORT = True

#: main endpoint to interact with AI Services
AI_SERVICE_ENDPOINT = "ai"

#: endpoint to manipulate AI Services data
AI_DATA_OP_ENDPOINT = "ai_data_op"


class AIResource(Resource):
    schema = {
        "service": {
            "type": "string",
            "required": True,
        },
        "item": {
            "type": "dict",
            "required": True,
            "schema": {
                "guid": {"type": "string", "required": True},
                "abstract": {"type": "string", "required": False},
                "language": {"type": "string", "required": True},
                "headline": {"type": "string", "nullable": True},
                "body_html": {"type": "string", "required": True},
            },
        },
    }
    datasource = {"projection": {"analysis": 1}}
    internal_resource = False
    resource_methods = ["POST"]
    item_methods = []


class AIService(BaseService):
    r"""Service managing article analysis with machine learning/AI related services

    When doing a POST request on this service, the following keys can be used (keys
    with a\* are required)

    ==========  ===========
    key         explanation
    ==========  ===========
    service \*  name of the service to use
    item \*     item metadata to be analyzed
    ==========  ===========

    e.g. to get autotagging with iMatrics service:

    .. sourcecode:: json

        {
            "service": "imatrics",
            "item": {
                "guid": "some_id",
                "headline": "item headline",
                "body_html": "item content"
            }
        }

    """

    def create(self, docs, **kwargs):
        doc = docs[0]
        service = doc["service"]
        item = doc["item"]
        try:
            service = registered_ai_services[service]
        except KeyError:
            raise SuperdeskApiError.notFoundError("{service} service can't be found".format(service=service))

        analyzed_data = service.analyze(item)
        docs[0].update({"analysis": analyzed_data})
        return [0]


class AIDataOpResource(Resource):
    schema = {
        "service": {
            "type": "string",
            "required": True,
        },
        "operation": {
            "type": "string",
            "required": True,
            "allowed": ["search", "create", "delete"],
        },
        "data_name": {
            "type": "string",
            "required": False,
        },
        "data": {
            "type": "dict",
            "required": True,
        },
    }
    datasource = {"projection": {"result": 1}}
    internal_resource = False
    resource_methods = ["POST"]
    item_methods = []


class AIDataOpService(BaseService):
    r"""Service to manipulate AI service related data

    When doing a POST request on this service, the following keys can be used (keys
    with a \* are required):


    ============  ===========
    key           explanation
    ============  ===========
    service \*    name of the service to use
    operation \*  operation that you want to do on the data (search, create, …)
    data_name     type of data to manipulate (empty for default or when service only
                  handle one type of data)
    data          argument of use for the operation
    ============  ===========

    e.g. to search for tags with iMatrics service:

    .. sourcecode:: json

        {
            "service": "imatrics",
            "operation": "search",
            "data": {"term": "some_term"},
        }
    """

    def create(self, docs, **kwargs):
        doc = docs[0]
        service = doc["service"]
        operation = doc["operation"]
        name = doc.get("data_name")
        data = doc["data"]
        try:
            service = registered_ai_services[service]
        except KeyError:
            raise SuperdeskApiError.notFoundError("{service} service can't be found".format(service=service))

        result = service.data_operation("POST", operation, name, data)
        docs[0].update({"result": result})
        return [0]


def init_app(app):
    if AUTO_IMPORT:
        tools.import_services(app, __name__, AIServiceBase)

    allowed_service = list(registered_ai_services)

    endpoint_name = AI_SERVICE_ENDPOINT
    service = AIService(endpoint_name, backend=superdesk.get_backend())
    AIResource.schema["service"]["allowed"] = allowed_service
    AIResource(endpoint_name, app=app, service=service)
    superdesk.intrinsic_privilege(endpoint_name, method=["POST"])

    endpoint_name = AI_DATA_OP_ENDPOINT
    service = AIDataOpService(endpoint_name, backend=superdesk.get_backend())
    AIDataOpResource.schema["service"]["allowed"] = allowed_service
    AIDataOpResource(endpoint_name, app=app, service=service)
    superdesk.intrinsic_privilege(endpoint_name, method=["POST"])



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


import os
from superdesk import config
from superdesk.tests import TestCase

from superdesk.etree import etree
from superdesk.io.feed_parsers.nitf import NITFFeedParser


class NITFTestCase(TestCase):

    vocab = [{"_id": "genre", "items": [{"name": "Current"}]}]

    def setUp(self):
        with self.app.app_context():
            self.app.data.insert("vocabularies", self.vocab)
        dirname = os.path.dirname(os.path.realpath(__file__))
        fixture = os.path.normpath(os.path.join(dirname, "../fixtures", self.filename))
        provider = {"name": "Test"}
        with open(fixture, "rb") as f:
            self.nitf = f.read()
            self.item = NITFFeedParser().parse(etree.fromstring(self.nitf), provider)


class AAPTestCase(NITFTestCase):

    filename = "aap.xml"

    def test_headline(self):
        self.assertEqual(self.item.get("headline"), "The main stories on today's 1900 ABC TV news")

    def test_slugline(self):
        self.assertEqual(self.item.get("slugline"), "Monitor 1900 ABC News")

    def test_subjects(self):
        self.assertEqual(len(self.item.get("subject")), 4)
        self.assertIn({"qcode": "02000000", "name": "Justice"}, self.item.get("subject"))
        self.assertIn({"qcode": "02003000", "name": "Police"}, self.item.get("subject"))
        self.assertIn({"qcode": "02003001", "name": "law enforcement"}, self.item.get("subject"))
        self.assertIn({"qcode": "02003002", "name": "investigation"}, self.item.get("subject"))

    def test_guid(self):
        self.assertEqual(self.item.get("guid"), "AAP.115314987.5417374")
        self.assertEqual(self.item.get("guid"), self.item.get("uri"))

    def test_type(self):
        self.assertEqual(self.item.get("type"), "text")

    def test_urgency(self):
        self.assertEqual(self.item.get("urgency"), 5)

    def test_dateline(self):
        self.assertEqual(self.item.get("dateline", {}).get("located", {}).get("city"), "Sydney")

    def test_byline(self):
        self.assertEqual(self.item.get("byline"), "By John Doe")

    def test_abstract(self):
        self.assertEqual(self.item.get("abstract"), "The main stories on today's 1900 ABC TV news")

    def test_dates(self):
        self.assertEqual(self.item.get("firstcreated").isoformat(), "2013-10-20T08:27:51+00:00")
        self.assertEqual(self.item.get("versioncreated").isoformat(), "2013-10-20T08:27:51+00:00")

    def test_content(self):
        text = "<p>   1A) More extreme weather forecast over the next few days the <br/>fire situation is likely"
        self.assertIn(text, self.item.get("body_html"))
        self.assertIsInstance(self.item.get("body_html"), type(""))
        self.assertNotIn("<body.content>", self.item.get("body_html"))

    def test_pubstatus(self):
        self.assertEqual("usable", self.item.get("pubstatus"))

    def test_ingest_provider_sequence(self):
        self.assertEqual(self.item.get("ingest_provider_sequence"), "1747")

    def test_anpa_category(self):
        self.assertEqual(self.item.get("anpa_category")[0]["qcode"], "a")

    def test_word_count(self):
        self.assertIsInstance(self.item.get("word_count"), int)


class APExampleTestCase(NITFTestCase):

    filename = "ap-nitf.xml"

    def test_headline(self):
        self.assertEqual(self.item.get("headline"), "Can trading pollution like stocks help fight climate change?")

    def test_byline(self):
        self.assertEqual(self.item.get("byline"), "By BERNARD CONDON")

    def test_ednote(self):
        self.assertEqual(self.item.get("ednote"), "For global distribution")

    def test_word_count(self):
        self.assertIsInstance(self.item.get("word_count"), int)


class IPTCExampleTestCase(NITFTestCase):

    filename = "nitf-fishing.xml"

    def test_headline(self):
        self.assertEqual(self.item.get("headline"), "Weather and Tide Updates for Norfolk")

    def test_pubstatus(self):
        self.assertEqual("canceled", self.item.get("pubstatus"))

    def test_guid(self):
        self.assertEqual("iptc.321656141.b", self.item.get("guid"))

    def test_subjects(self):
        self.assertEqual(3, len(self.item.get("subject")))

    def test_place(self):
        places = self.item.get("place", [])
        self.assertEqual(1, len(places))
        self.assertEqual("Norfolk", places[0]["name"])
        self.assertEqual("US", places[0]["code"])
        self.assertEqual("Norfolk", places[0]["qcode"])

    def test_expiry(self):
        self.assertEqual("2012-02-26T14:30:00+00:00", self.item.get("expiry").isoformat())

    def test_keywords(self):
        self.assertIn("fishing", self.item.get("keywords"))

    def test_ednote(self):
        self.assertEqual("Today begins an expanded format of this popular column.", self.item.get("ednote"))

    def test_byline(self):
        self.assertEqual("By Alan Karben", self.item.get("byline"))

    def test_word_count(self):
        self.assertIsInstance(self.item.get("word_count"), int)


class PATestCase(NITFTestCase):

    filename = "pa1.xml"

    def test_headline(self):
        self.assertEqual(self.item.get("headline"), "PA SPORT TRIVIA (OCTOBER 14)")

    def test_pubstatus(self):
        self.assertEqual("usable", self.item.get("pubstatus"))

    def test_guid(self):
        self.assertEqual("af1f7ad5-5619-49de-84cc-2e608538c77fSSS-3-1", self.item.get("guid"))
        self.assertEqual(self.item.get("format"), "HTML")

    def test_subjects(self):
        self.assertEqual(4, len(self.item.get("subject")))

    def test_keywords(self):
        self.assertIn("Trivia (Oct 14)", self.item.get("keywords"))

    def test_word_count(self):
        self.assertIsInstance(self.item.get("word_count"), int)


class PATestCase2(NITFTestCase):

    filename = "pa2.xml"

    def test_headline(self):
        self.assertEqual(self.item.get("headline"), "1 SOCCER INT-Teams")

    def test_pubstatus(self):
        self.assertEqual("usable", self.item.get("pubstatus"))

    def test_guid(self):
        self.assertEqual("T201510140143580001T", self.item.get("guid"))
        self.assertEqual(self.item.get("format"), "preserved")

    def test_guid_2(self):
        self.assertEqual(self.item.get("type"), "text")

    def test_subjects(self):
        self.assertEqual(0, len(self.item.get("subject")))

    def test_keywords(self):
        self.assertIn("INT-Teams", self.item.get("keywords"))

    def test_word_count(self):
        self.assertEqual(58, self.item.get("word_count"))


class ParseSubjects(TestCase):
    def test_get_subjects(self):
        xml = (
            b'<?xml version="1.0" encoding="UTF-8"?>'
            b"<nitf><head>"
            b'<tobject tobject.type="News">'
            b'<tobject.property tobject.property.type="Current" />'
            b'<tobject.subject tobject.subject.refnum="02003000" '
            b'tobject.subject.type="Justice" tobject.subject.matter="Police" />'
            b"</tobject></head></nitf>"
        )
        subjects = NITFFeedParser().get_subjects(etree.fromstring(xml))
        self.assertEqual(len(subjects), 2)
        self.assertIn({"qcode": "02000000", "name": "Justice"}, subjects)
        self.assertIn({"qcode": "02003000", "name": "Police"}, subjects)

    def test_get_subjects_with_invalid_qcode(self):
        xml = (
            b'<?xml version="1.0" encoding="UTF-8"?>'
            b"<nitf><head>"
            b'<tobject tobject.type="News">'
            b'<tobject.property tobject.property.type="Current" />'
            b'<tobject.subject tobject.subject.refnum="00000000" '
            b'tobject.subject.type="Justice" tobject.subject.matter="Police" />'
            b"</tobject></head></nitf>"
        )
        subjects = NITFFeedParser().get_subjects(etree.fromstring(xml))
        self.assertEqual(len(subjects), 0)


class MappingTestCase(TestCase):

    filename = "mapping_test.xml"
    mapping = {
        "subject": {"update": True, "key_hook": lambda item, value: item.setdefault("subject", []).extend(value)},
        "subject_test": {
            "callback": lambda _: ["TEST OK"],
            "key_hook": lambda item, value: item.setdefault("subject", []).extend(value),
        },
    }

    def setUp(self):
        config.NITF_MAPPING = self.mapping
        dirname = os.path.dirname(os.path.realpath(__file__))
        fixture = os.path.normpath(os.path.join(dirname, "../fixtures", self.filename))
        provider = {"name": "Test"}
        with open(fixture, "rb") as f:
            self.nitf = f.read()
            self.item = NITFFeedParser().parse(etree.fromstring(self.nitf), provider)

    def test_update_and_hook(self):
        subjects = self.item.get("subject")
        # have we got both items ?
        self.assertEqual(len(subjects), 2)
        # the initial updated subject need to be here
        self.assertIn({"qcode": "02000000", "name": "Kriminalitet og rettsvesen"}, subjects)
        # and our key from subject_test need to be here too
        self.assertIn("TEST OK", subjects)

    def tearDown(self):
        del config.NITF_MAPPING


class HandleInvalidFieldsTestCase(NITFTestCase):

    filename = "handle_invalid_fields.xml"

    def test_dates(self):
        self.assertEqual(self.item.get("firstcreated").isoformat(), "2020-07-03T20:51:36+00:00")
        self.assertEqual(self.item.get("versioncreated").isoformat(), "2020-07-03T20:51:36+00:00")

    def test_expiry(self):
        self.assertEqual(self.item.get("expiry"), None)

    def test_keywords(self):
        self.assertEqual(len(self.item.get("keywords")), 0)


class NTBTestCase(NITFTestCase):

    filename = "nitf-ntb.xml"

    def test_place(self):
        self.assertEqual([], self.item.get("place"))



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014, 2015 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

"""
A module containing configuration of the Superdesk's public API.

The meaning of configuration options is described in the Eve framework
`documentation <http://python-eve.org/config.html#global-configuration>`_.
"""

from superdesk.default_settings import env, urlparse

from superdesk.default_settings import (  # noqa
    SECRET_KEY,
    MONGO_DBNAME,
    MONGO_URI,
    CONTENTAPI_MONGO_DBNAME,
    CONTENTAPI_MONGO_URI,
    CONTENTAPI_ELASTICSEARCH_URL,
    CONTENTAPI_ELASTICSEARCH_INDEX,
    AMAZON_ACCESS_KEY_ID,
    AMAZON_SECRET_ACCESS_KEY,
    AMAZON_REGION,
    AMAZON_CONTAINER_NAME,
    AMAZON_S3_SUBFOLDER,
    AMAZON_OBJECT_ACL,
    AMAZON_ENDPOINT_URL,
)

CONTENTAPI_INSTALLED_APPS = [
    "content_api.items",
    "content_api.packages",
    "content_api.assets",
    "content_api.tokens",
    "content_api.items_versions",
    "content_api.packages_versions",
    "content_api.api_audit",
]

CONTENTAPI_DOMAIN = {}

# NOTE: no trailing slash for the CONTENTAPI_URL setting!
CONTENTAPI_URL = env("CONTENTAPI_URL", "http://localhost:5400")
MEDIA_PREFIX = env("MEDIA_PREFIX", "%s/assets" % CONTENTAPI_URL.rstrip("/"))
server_url = urlparse(CONTENTAPI_URL)
URL_PREFIX = env("CONTENTAPI_URL_PREFIX", server_url.path.strip("/")) or ""
VERSION = "_current_version"

PUBLIC_RESOURCES = []
DATE_FORMAT = "%Y-%m-%dT%H:%M:%S+0000"
ELASTIC_DATE_FORMAT = "%Y-%m-%d"
BCRYPT_GENSALT_WORK_FACTOR = 12



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from typing import NamedTuple
from copy import deepcopy

from superdesk.resource import Resource, not_analyzed, not_indexed, not_enabled, text_with_keyword
from .packages import LINKED_IN_PACKAGES, PACKAGE
from eve.utils import config
from superdesk.utils import SuperdeskBaseEnum

GUID_TAG = "tag"
GUID_FIELD = "guid"
GUID_NEWSML = "newsml"
INGEST_ID = "ingest_id"
INGEST_VERSION = "ingest_version"
FAMILY_ID = "family_id"
ASSOCIATIONS = "associations"


#: item public states
class PubStatuses(NamedTuple):
    USABLE: str
    HOLD: str
    CANCELED: str


PUB_STATUS: PubStatuses = PubStatuses("usable", "withheld", "canceled")


class ContentTypes(NamedTuple):
    TEXT: str
    PREFORMATTED: str
    AUDIO: str
    VIDEO: str
    PICTURE: str
    GRAPHIC: str
    COMPOSITE: str
    EVENT: str


CONTENT_TYPE: ContentTypes = ContentTypes(
    "text", "preformatted", "audio", "video", "picture", "graphic", "composite", "event"
)

MEDIA_TYPES = ("audio", "video", "picture", "graphic")
ITEM_TYPE = "type"
ITEM_STATE = "state"
ITEM_PRIORITY = "priority"
ITEM_URGENCY = "urgency"


#: item internal states
class ContentStates(NamedTuple):
    DRAFT: str
    INGESTED: str
    ROUTED: str
    FETCHED: str
    SUBMITTED: str
    PROGRESS: str
    SPIKED: str
    PUBLISHED: str
    KILLED: str
    CORRECTED: str
    SCHEDULED: str
    RECALLED: str
    UNPUBLISHED: str
    CORRECTION: str
    BEING_CORRECTED: str


CONTENT_STATE: ContentStates = ContentStates(
    "draft",
    "ingested",
    "routed",
    "fetched",
    "submitted",
    "in_progress",
    "spiked",
    "published",
    "killed",
    "corrected",
    "scheduled",
    "recalled",
    "unpublished",
    "correction",
    "being_corrected",
)

PUBLISH_STATES = {
    CONTENT_STATE.PUBLISHED,
    CONTENT_STATE.SCHEDULED,
    CONTENT_STATE.CORRECTED,
    CONTENT_STATE.KILLED,
    CONTENT_STATE.RECALLED,
    CONTENT_STATE.UNPUBLISHED,
    CONTENT_STATE.BEING_CORRECTED,
}


class Formats(NamedTuple):
    HTML: str
    PRESERVED: str


FORMAT = "format"
FORMATS: Formats = Formats("HTML", "preserved")

BYLINE = "byline"
SIGN_OFF = "sign_off"
EMBARGO = "embargo"
PUBLISH_SCHEDULE = "publish_schedule"
SCHEDULE_SETTINGS = "schedule_settings"
PROCESSED_FROM = "processed_from"

# part the task dict
LAST_DESK = "last_desk"
LAST_AUTHORING_DESK = "last_authoring_desk"
LAST_PRODUCTION_DESK = "last_production_desk"
DESK_HISTORY = "desk_history"

ITEM_EVENT_ID = "event_id"

geopoint = {
    "type": "dict",
    "mapping": {"type": "geo_point"},
    "nullable": True,
    "schema": {
        "lat": {"type": "float"},
        "lon": {"type": "float"},
    },
}

entity_metadata = {
    "type": "list",
    "nullable": True,
    "mapping": {
        "type": "object",
        "dynamic": False,
        "properties": {
            "name": text_with_keyword,
            "qcode": not_analyzed,
            "scheme": not_analyzed,
            "source": not_analyzed,
        },
    },
}

metadata_schema = {
    config.ID_FIELD: {"type": "string", "unique": True},
    #: Identifiers
    "guid": {"type": "string", "unique": True, "mapping": not_analyzed},
    "uri": {
        "type": "string",
        "mapping": not_analyzed,
    },
    "unique_id": {
        "type": "integer",
        "unique": True,
    },
    "unique_name": {"type": "string", "unique": True, "mapping": not_analyzed},
    "version": {"type": "integer"},
    "ingest_id": {"type": "string", "mapping": not_analyzed},
    "ingest_version": {"type": "string", "mapping": not_analyzed},
    "family_id": {"type": "string", "mapping": not_analyzed},
    "related_to": {  # this field keeps a reference to the related item from which metadata has been copied
        "type": "string",
        "mapping": not_analyzed,
    },
    # Audit Information
    "original_creator": Resource.rel("users"),
    "version_creator": Resource.rel("users"),
    "firstcreated": {"type": "datetime"},
    "versioncreated": {"type": "datetime"},
    "firstpublished": {
        "type": "datetime",
        "required": False,
        "nullable": True,
    },
    # Ingest Details
    "ingest_provider": Resource.rel("ingest_providers"),
    "source": {"type": "string", "mapping": not_analyzed},  # The value is copied from the ingest_providers vocabulary
    "original_source": {"type": "string", "mapping": not_analyzed},  # This value is extracted from the ingest
    "ingest_provider_sequence": {"type": "string", "mapping": not_analyzed},
    # Copyright Information
    "usageterms": {
        "type": "string",
        "nullable": True,
    },
    "copyrightnotice": {"type": "string", "nullable": True, "mapping": not_indexed},
    "copyrightholder": {"type": "string", "nullable": True},
    # Category Details
    "anpa_category": {
        "type": "list",
        "nullable": True,
        "mapping": {
            "type": "object",
            "properties": {
                "qcode": not_analyzed,
                "name": not_analyzed,
            },
        },
    },
    "subject": {
        "type": "list",
        "mapping": {"type": "object", "dynamic": False, "properties": {"qcode": not_analyzed, "name": not_analyzed}},
    },
    "genre": {
        "type": "list",
        "nullable": True,
        "mapping": {"type": "object", "properties": {"name": not_analyzed, "qcode": not_analyzed}},
    },
    "company_codes": {
        "type": "list",
        "mapping": {
            "type": "object",
            "properties": {"qcode": not_analyzed, "name": not_analyzed, "security_exchange": not_analyzed},
        },
    },
    # Item Metadata
    ITEM_TYPE: {
        "type": "string",
        "allowed": tuple(CONTENT_TYPE),
        "default": "text",
        "mapping": not_analyzed,
    },
    "package_type": {"type": "string", "allowed": ["takes"]},  # deprecated
    "language": {
        "type": "string",
        "mapping": not_analyzed,
        "nullable": True,
    },
    "abstract": {
        "type": "string",
        "nullable": True,
    },
    "headline": {"type": "string"},
    "slugline": {
        "type": "string",
        "mapping": {
            "type": "string",
            "fielddata": True,
            "fields": {
                "phrase": {
                    "type": "string",
                    "analyzer": "phrase_prefix_analyzer",
                    "search_analyzer": "phrase_prefix_analyzer",
                    "fielddata": True,
                }
            },
        },
    },
    "anpa_take_key": {
        "type": "string",
        "nullable": True,
    },
    "correction_sequence": {"type": "integer", "nullable": True, "mapping": not_analyzed},
    "rewrite_sequence": {"type": "integer", "nullable": True, "mapping": not_analyzed},
    "rewrite_of": {
        "type": "string",
        "nullable": True,
        "mapping": not_analyzed,
    },
    "rewritten_by": {
        "type": "string",
        "nullable": True,
        "mapping": not_analyzed,
    },
    "sequence": {
        "type": "integer",
        "nullable": True,
    },
    "keywords": {"type": "list", "mapping": {"type": "string"}},
    "word_count": {"type": "integer"},
    "priority": {"type": "integer", "nullable": True},
    "urgency": {"type": "integer", "nullable": True},
    "profile": {
        "type": "string",
        "nullable": True,
        "mapping": not_analyzed,
    },
    # Related to state of an article
    ITEM_STATE: {
        "type": "string",
        "allowed": tuple(CONTENT_STATE),
        "mapping": not_analyzed,
    },
    # The previous state the item was in before for example being spiked, when un-spiked it will revert to this state
    "revert_state": {
        "type": "string",
        "allowed": tuple(CONTENT_STATE),
        "mapping": not_analyzed,
    },
    "pubstatus": {
        "type": "string",
        "allowed": tuple(PUB_STATUS),
        "default": PUB_STATUS.USABLE,
        "mapping": not_analyzed,
        "nullable": True,
    },
    "signal": {
        "type": "list",
        "mapping": {
            "type": "object",
            "properties": {"qcode": not_analyzed, "name": not_analyzed, "scheme": not_analyzed},
        },
    },
    BYLINE: {
        "type": "string",
        "nullable": True,
    },
    "ednote": {
        "type": "string",
        "nullable": True,
    },
    "authors": {
        "type": "list",
        "nullable": True,
        "mapping": {
            "type": "object",
            "dynamic": False,
            "properties": {
                "uri": not_analyzed,
                "parent": not_analyzed,
                "name": not_analyzed,
                "role": not_analyzed,
                "jobtitle": not_enabled,
            },
        },
    },
    "description_text": {"type": "string", "nullable": True},
    # This is a description of the item as recieved from its source.
    "archive_description": {"type": "string", "nullable": True},
    "groups": {
        "type": "list",
        "minlength": 1,
        "nullable": True,
        "mapping": {
            "dynamic": False,
            "properties": {
                "id": not_analyzed,
                "refs": {
                    "dynamic": False,
                    "properties": {
                        "idRef": not_analyzed,
                        "_id": not_analyzed,
                        "uri": not_analyzed,
                        "guid": not_analyzed,
                        "type": not_analyzed,
                        "location": not_analyzed,
                        "headline": {"type": "string"},
                        "slugline": {"type": "string"},
                    },
                },
            },
        },
    },
    "deleted_groups": {
        "type": "list",
        "minlength": 1,
        "nullable": True,
    },
    "body_html": {
        "type": "string",
        "nullable": True,
        "mapping": {"type": "string", "analyzer": "html_field_analyzer", "search_analyzer": "standard"},
    },
    "body_text": {
        "type": "string",
        "nullable": True,
    },
    "dateline": {
        "type": "dict",
        "nullable": True,
        "schema": {
            "located": {
                "type": "dict",
                "nullable": True,
                "schema": {
                    "state_code": {"type": "string"},
                    "city": {"type": "string"},
                    "tz": {"type": "string"},
                    "country_code": {"type": "string"},
                    "dateline": {"type": "string"},
                    "alt_name": {"type": "string"},
                    "state": {"type": "string"},
                    "city_code": {"type": "string"},
                    "country": {"type": "string"},
                    "code": {"type": "string"},
                    "scheme": {"type": "string"},
                    "location": geopoint,
                    "place": {
                        "type": "dict",
                        "nullable": True,
                        "mapping": not_enabled,
                        "schema": {
                            "code": {"type": "string"},
                            "name": {"type": "string"},
                            "qcode": {"type": "string"},
                            "scheme": {"type": "string"},
                            "feature_class": {"type": "string"},
                            "location": geopoint,
                            "continent_code": {"type": "string", "nullable": True},
                            "region": {"type": "string", "nullable": True},
                            "region_code": {"type": "string", "nullable": True},
                            "locality": {"type": "string", "nullable": True},
                            "state": {"type": "string", "nullable": True},
                            "country": {"type": "string", "nullable": True},
                            "world_region": {"type": "string", "nullable": True},
                            "locality_code": {"type": "string", "nullable": True},
                            "state_code": {"type": "string", "nullable": True},
                            "country_code": {"type": "string", "nullable": True},
                            "world_region_code": {"type": "string", "nullable": True},
                            "rel": {"type": "string", "nullable": True},
                            "tz": {"type": "string", "nullable": True},
                        },
                    },
                },
            },
            "date": {"type": "datetime", "nullable": True},
            "source": {"type": "string"},
            "text": {"type": "string", "nullable": True},
        },
    },
    "expiry": {"type": "datetime"},
    # Media Related
    "media": {"type": "file"},
    "mimetype": {"type": "string", "mapping": not_analyzed},
    "poi": {
        "type": "dict",
        "schema": {"x": {"type": "float", "nullable": False}, "y": {"type": "float", "nullable": False}},
    },
    "renditions": {
        "type": "dict",
        "schema": {},
        "allow_unknown": True,
        "mapping": not_enabled,
    },
    "filemeta": {
        "type": "dict",
        "schema": {},
        "allow_unknown": True,
        "mapping": not_enabled,
    },
    "filemeta_json": {"type": "string"},
    "media_file": {"type": "string"},
    "contents": {"type": "list"},
    ASSOCIATIONS: {
        "type": "dict",
        "allow_unknown": True,
        "schema": {},
        "mapping": {
            "type": "object",
            "dynamic": False,
            "properties": {
                "featuremedia": {  # keep indexing featuremedia - we do some filtering using it
                    "type": "object",
                    "dynamic": False,
                    "properties": {
                        "_id": not_analyzed,
                        "guid": not_analyzed,
                        "unique_id": {"type": "integer"},
                    },
                }
            },
        },
    },
    # track references to other objects,
    # based on associations but allows queries
    "refs": {
        "type": "list",
        "readonly": True,
        "schema": {
            "_id": {"type": "string"},
            "key": {"type": "string"},
            "uri": {"type": "string"},
            "guid": {"type": "string"},
            "type": {"type": "string"},
        },
        "mapping": {
            "type": "object",
            "properties": {
                "_id": not_analyzed,
                "key": not_analyzed,
                "uri": not_analyzed,
                "guid": not_analyzed,
                "type": not_analyzed,
            },
        },
    },
    "alt_text": {"type": "string", "nullable": True},
    # aka Locator as per NewML Specification
    "place": {
        "type": "list",
        "nullable": True,
        "mapping": {
            "type": "object",
            "dynamic": False,
            "properties": {
                "scheme": not_analyzed,
                "qcode": not_analyzed,
                "code": not_analyzed,  # content api
                "name": not_analyzed,
                "locality": not_analyzed,  # can be used for city/town/village etc.
                "state": not_analyzed,
                "country": not_analyzed,
                "world_region": not_analyzed,
                "locality_code": not_analyzed,
                "state_code": not_analyzed,
                "country_code": not_analyzed,
                "world_region_code": not_analyzed,
                "feature_class": not_analyzed,
                "location": {"type": "geo_point"},
                "rel": not_analyzed,
            },
        },
    },
    "event": deepcopy(entity_metadata),
    "person": deepcopy(entity_metadata),
    "object": deepcopy(entity_metadata),
    "organisation": deepcopy(entity_metadata),
    # Not Categorized
    "creditline": {"type": "string"},
    LINKED_IN_PACKAGES: {
        "type": "list",
        "readonly": True,
        "schema": {
            "type": "dict",
            "schema": {PACKAGE: Resource.rel("archive"), "package_type": {"type": "string"}},  # deprecated
        },
    },
    "highlight": Resource.rel("highlights"),
    "highlights": {"type": "list", "schema": Resource.rel("highlights", True)},
    "marked_desks": {
        "type": "list",
        "nullable": True,
        "schema": {
            "type": "dict",
            "schema": {
                "desk_id": Resource.rel("desks", True),
                "date_marked": {"type": "datetime", "nullable": True},
                "user_marked": Resource.rel("users", required=False, nullable=True),
                "date_acknowledged": {"type": "datetime", "nullable": True},
                "user_acknowledged": Resource.rel("users", required=False, nullable=True),
            },
        },
    },
    "more_coming": {"type": "boolean"},  # deprecated
    # Field which contains all the sign-offs done on this article, eg. twd/jwt/ets
    SIGN_OFF: {
        "type": "string",
        "nullable": True,
    },
    # Desk and Stage Details
    "task": {
        "type": "dict",
        "schema": {
            "user": {"type": "string", "mapping": not_analyzed, "nullable": True},
            "desk": {"type": "string", "mapping": not_analyzed, "nullable": True},
            "desk_history": {"type": "list", "mapping": not_analyzed},
            "last_desk": {"type": "string", "mapping": not_analyzed},
            "stage": {"type": "string", "mapping": not_analyzed, "nullable": True},
            "status": {"type": "string", "mapping": not_analyzed},
        },
    },
    # Task and Lock Details
    "task_id": {"type": "string", "mapping": not_analyzed, "versioned": False},
    "lock_user": Resource.rel("users"),
    "lock_time": {"type": "datetime", "versioned": False},
    "lock_session": Resource.rel("auth"),
    # Action when the story is locked: edit, correct, kill
    "lock_action": {"type": "string", "mapping": not_analyzed, "nullable": True},
    # template used to create an item
    "template": Resource.rel("content_templates"),
    "body_footer": {  # Public Service Announcements
        "type": "string",
        "nullable": True,
        "mapping": not_indexed,
    },
    "flags": {
        "type": "dict",
        "schema": {
            "marked_for_not_publication": {"type": "boolean", "default": False},
            "marked_for_legal": {"type": "boolean", "default": False},
            "marked_archived_only": {"type": "boolean", "default": False},
            "marked_for_sms": {"type": "boolean", "default": False},
        },
        "default": {
            "marked_for_not_publication": False,
            "marked_for_legal": False,
            "marked_archived_only": False,
            "marked_for_sms": False,
        },
    },
    "sms_message": {"type": "string", "mapping": not_analyzed, "nullable": True},
    FORMAT: {"type": "string", "mapping": not_analyzed, "default": FORMATS.HTML},
    # True indicates that the item has been or is to be published as a result of a routing rule
    "auto_publish": {"type": "boolean"},
    # draft-js internal data
    "fields_meta": {
        "type": "dict",
        "schema": {},
        "allow_unknown": True,
        "nullable": True,
        "mapping": not_enabled,
    },
    "annotations": {
        "type": "list",
        "mapping": not_enabled,
        "schema": {
            "type": "dict",
            "schema": {
                "id": {"type": "integer"},
                "type": {"type": "string"},
                "body": {"type": "string"},
            },
        },
    },
    "extra": {
        "type": "dict",
        "schema": {},
        "mapping": not_enabled,
        "allow_unknown": True,
    },
    "attachments": {
        "type": "list",
        "nullable": True,
        "schema": {
            "type": "dict",
            "schema": {
                "attachment": Resource.rel("attachments", nullable=False),
            },
        },
    },
    # references assignment related to the coverage
    "assignment_id": {"type": "string", "mapping": not_analyzed},
    "translated_from": {
        "type": "string",
        "mapping": not_analyzed,
    },
    "translation_id": {
        "type": "string",
        "mapping": not_analyzed,
    },
    "translations": {
        "type": "list",
        "mapping": not_analyzed,
    },
    # references item id for items auto published using internal destinations
    PROCESSED_FROM: {"type": "string", "mapping": not_analyzed},
    # ingested embargoed info, not using embargo to avoid validation
    "embargoed": {"type": "datetime"},
    "embargoed_text": {"type": "string", "mapping": not_indexed},
    "marked_for_user": Resource.rel("users", required=False, nullable=True),
    "broadcast": {
        "type": "dict",
        "schema": {
            "status": {"type": "string", "mapping": not_analyzed},
            "master_id": {"type": "string", "mapping": not_analyzed},
            "rewrite_id": {"type": "string", "mapping": not_analyzed},
        },
    },
    ITEM_EVENT_ID: {"type": "string", "mapping": not_analyzed},
    # schedules
    EMBARGO: {"type": "datetime", "nullable": True},
    PUBLISH_SCHEDULE: {"type": "datetime", "nullable": True},
    SCHEDULE_SETTINGS: {
        "type": "dict",
        "schema": {
            "time_zone": {"type": "string", "nullable": True, "mapping": not_analyzed},
            "utc_embargo": {"type": "datetime", "nullable": True},
            "utc_publish_schedule": {"type": "datetime", "nullable": True},
        },
    },
    # usage tracking
    "used": {"type": "boolean"},
    "used_count": {"type": "integer"},
    "used_updated": {"type": "datetime"},
    # system fields
    "_type": {"type": "string", "mapping": None},
    "operation": {"type": "string"},
    "es_highlight": {"type": "dict", "allow_unknown": True, "readonly": True},
    # targeting fields
    "target_regions": {
        "type": "list",
        "nullable": True,
        "schema": {
            "type": "dict",
            "schema": {"qcode": {"type": "string"}, "name": {"type": "string"}, "allow": {"type": "boolean"}},
        },
    },
    "target_types": {
        "type": "list",
        "nullable": True,
        "schema": {
            "type": "dict",
            "schema": {"qcode": {"type": "string"}, "name": {"type": "string"}, "allow": {"type": "boolean"}},
        },
    },
    "target_subscribers": {"type": "list", "nullable": True},
}

metadata_schema["lock_user"]["versioned"] = False
metadata_schema["lock_session"]["versioned"] = False

crop_schema = {
    "CropLeft": {"type": "integer"},
    "CropRight": {"type": "integer"},
    "CropTop": {"type": "integer"},
    "CropBottom": {"type": "integer"},
}


def remove_metadata_for_publish(item):
    """Remove metadata from item that should not be public.

    :param item: Item containing the metadata
    :return: item
    """
    from superdesk.attachments import is_attachment_public

    if len(item.get("attachments", [])) > 0:
        item["attachments"] = [attachment for attachment in item["attachments"] if is_attachment_public(attachment)]

    return item


class Priority(SuperdeskBaseEnum):
    """Priority values."""

    Flash = 1
    Urgent = 2
    Three_Paragraph = 3
    Screen_Finance = 4
    Continuous_News = 5
    Ordinary = 6


def get_schema(versioning=False):
    schema = metadata_schema.copy()

    if versioning:
        schema.update(
            {
                "_id_document": {"type": "string"},
                "_current_version": {"type": "integer"},
            }
        )

    return schema



from superdesk.tests import TestCase
from apps.rules.routing_rules import RoutingRuleSchemeService


class SetDefaultValuesTestCase(TestCase):
    def test_setting_default_values(self):

        instance = RoutingRuleSchemeService()
        category = [{"qcode": "a", "name": "Australian General News"}]
        result = instance._assign_default_values({"anpa_category": category}, None)
        self.assertEqual(result["anpa_category"], [{"qcode": "a", "name": "Australian General News"}])

        with self.app.app_context():
            result = instance._assign_default_values({}, None)
            self.assertIsNone(result["anpa_category"])

            result = instance._assign_default_values({}, [{"qcode": "a", "name": "Australian General News"}])
            self.assertEqual(result["anpa_category"], [{"qcode": "a", "name": "Australian General News"}])

    def test_getting_selected_categories(self):
        vocabularies = [
            {
                "_id": "categories",
                "items": [
                    {"qcode": "a", "name": "foo", "is_active": True},
                    {"qcode": "b", "name": "bar", "is_active": True},
                    {"qcode": "c", "name": "baz", "is_active": False},
                ],
            }
        ]

        instance = RoutingRuleSchemeService()

        with self.app.app_context():
            self.app.data.insert("vocabularies", vocabularies)
            result = instance._get_categories("a")
            self.assertEqual(result, [{"qcode": "a", "name": "foo"}])

            result = instance._get_categories("a,b")
            self.assertEqual(result, [{"qcode": "a", "name": "foo"}, {"qcode": "b", "name": "bar"}])

            result = instance._get_categories("a,c")
            self.assertEqual(result, [{"qcode": "a", "name": "foo"}])

            result = instance._get_categories("c")
            self.assertEqual(result, [])

            result = instance._get_categories(None)
            self.assertIsNone(result)



from superdesk import SearchProvider


class TestSearchProvider(SearchProvider):

    label = "Foo"

    def find(self, query):
        return [{"guid": "foo", "source": "bar"}]

    def fetch(self, guid):
        return {"_id": guid, "headline": guid, "source": "bar"}

    def fetch_file(self, href):
        return {"href": href}



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


import copy
from datetime import timedelta
from unittest.mock import MagicMock, patch

from apps.archive.archive import SOURCE as ARCHIVE
from apps.archive.common import remove_media_files
from superdesk import get_resource_service
from superdesk.media.crop import CropService
from superdesk.tests import TestCase
from superdesk.utc import get_expiry_date, utcnow


NOW = utcnow()


class RemoveSpikedContentTestCase(TestCase):
    articles = [
        {
            "guid": "tag:localhost:2015:69b961ab-2816-4b8a-a584-a7b402fed4f9",
            "_id": "1",
            "type": "text",
            "last_version": 3,
            "_current_version": 4,
            "body_html": "Test body",
            "urgency": 4,
            "headline": "Two students missing",
            "pubstatus": "usable",
            "firstcreated": utcnow(),
            "byline": "By Alan Karben",
            "ednote": "Andrew Marwood contributed to this article",
            "keywords": ["Student", "Crime", "Police", "Missing"],
            "subject": [{"qcode": "17004000", "name": "Statistics"}, {"qcode": "04001002", "name": "Weather"}],
            "state": "draft",
            "expiry": utcnow() + timedelta(minutes=20),
            "unique_name": "#1",
        },
        {
            "guid": "tag:localhost:2015:69b961ab-2816-4b8a-a974-xy4532fe33f9",
            "_id": "2",
            "last_version": 3,
            "_current_version": 4,
            "body_html": "Test body of the second article",
            "urgency": 4,
            "headline": "Another two students missing",
            "pubstatus": "usable",
            "firstcreated": utcnow(),
            "byline": "By Alan Karben",
            "ednote": "Andrew Marwood contributed to this article",
            "keywords": ["Student", "Crime", "Police", "Missing"],
            "subject": [{"qcode": "17004000", "name": "Statistics"}, {"qcode": "04001002", "name": "Weather"}],
            "expiry": utcnow() + timedelta(minutes=20),
            "state": "draft",
            "type": "text",
            "unique_name": "#2",
        },
        {
            "guid": "tag:localhost:2015:69b961ab-2816-4b8a-a584-a7b402fed4fa",
            "_id": "3",
            "_current_version": 4,
            "body_html": "Test body",
            "urgency": 4,
            "headline": "Two students missing killed",
            "pubstatus": "usable",
            "firstcreated": utcnow(),
            "byline": "By Alan Karben",
            "ednote": "Andrew Marwood contributed to this article killed",
            "keywords": ["Student", "Crime", "Police", "Missing"],
            "subject": [{"qcode": "17004000", "name": "Statistics"}, {"qcode": "04001002", "name": "Weather"}],
            "state": "draft",
            "expiry": utcnow() + timedelta(minutes=20),
            "type": "text",
            "unique_name": "#3",
        },
        {
            "guid": "tag:localhost:2015:69b961ab-2816-4b8a-a584-a7b402fed4fc",
            "_id": "4",
            "_current_version": 3,
            "state": "draft",
            "type": "composite",
            "groups": [
                {"id": "root", "refs": [{"idRef": "main"}], "role": "grpRole:NEP"},
                {
                    "id": "main",
                    "refs": [
                        {"location": "archive", "guid": "1", "residRef": "1", "type": "text"},
                        {"location": "archive", "residRef": "2", "guid": "2", "type": "text"},
                    ],
                    "role": "grpRole:main",
                },
            ],
            "firstcreated": utcnow(),
            "expiry": utcnow() + timedelta(minutes=20),
            "unique_name": "#4",
        },
        {
            "guid": "tag:localhost:2015:69b961ab-4b8a-a584-2816-a7b402fed4fc",
            "_id": "5",
            "_current_version": 3,
            "state": "draft",
            "type": "composite",
            "groups": [
                {"id": "root", "refs": [{"idRef": "main"}, {"idRef": "story"}], "role": "grpRole:NEP"},
                {
                    "id": "main",
                    "refs": [{"location": "archive", "guid": "1", "residRef": "1", "type": "text"}],
                    "role": "grpRole:main",
                },
                {
                    "id": "story",
                    "refs": [{"location": "archive", "guid": "4", "residRef": "4", "type": "composite"}],
                    "role": "grpRole:story",
                },
            ],
            "firstcreated": utcnow(),
            "expiry": utcnow() + timedelta(minutes=20),
            "unique_name": "#5",
        },
    ]

    media = {
        "viewImage": {
            "media": "1592730d582080f4e9fcc2fcf43aa357bda0ed19ffe314ee3248624cd4d4bc54",
            "mimetype": "image/jpeg",
            "href": "http://192.168.220.209/api/upload/abc/raw?_schema=http",
            "height": 452,
            "width": 640,
        },
        "thumbnail": {
            "media": "52250b4f37da50ee663fdbff057a5f064479f8a8bbd24fb8fdc06135d3f807bb",
            "mimetype": "image/jpeg",
            "href": "http://192.168.220.209/api/upload/abc/raw?_schema=http",
            "height": 120,
            "width": 169,
        },
        "baseImage": {
            "media": "7a608aa8f51432483918027dd06d0ef385b90702bfeba84ac4aec38ed1660b18",
            "mimetype": "image/jpeg",
            "href": "http://192.168.220.209/api/upload/abc/raw?_schema=http",
            "height": 990,
            "width": 1400,
        },
        "original": {
            "media": "stub.jpeg",
            "mimetype": "image/jpeg",
            "href": "http://192.168.220.209/api/upload/stub.jpeg/raw?_schema=http",
            "height": 2475,
            "width": 3500,
        },
    }

    def test_query_getting_expired_content(self):
        now = utcnow()

        self.app.data.insert(
            ARCHIVE,
            [
                {"expiry": get_expiry_date(0), "state": "spiked"},
                {"expiry": get_expiry_date(10), "state": "spiked"},
                {"expiry": get_expiry_date(20), "state": "spiked"},
                {"expiry": get_expiry_date(30), "state": "spiked"},
                {"expiry": None, "state": "spiked"},
                {"unique_id": 97, "state": "spiked"},
                {"expiry": now - timedelta(minutes=10), "state": "spiked", "unique_id": 100},
            ],
        )

        expired_items = get_resource_service(ARCHIVE).get_expired_items(now)
        now = utcnow()
        for expired_items in get_resource_service(ARCHIVE).get_expired_items(now):
            self.assertEquals(1, len(expired_items))
            self.assertEquals(100, expired_items[0]["unique_id"])

    def test_remove_media_files_for_picture(self):
        item = {"_id": "testimage", "type": "picture", "renditions": self.media}

        original = item.copy()
        with patch.object(self.app.media, "delete") as media_delete:
            CropService().update_media_references(item, original)
            references_service = get_resource_service("media_references")
            refs = references_service.get(req=None, lookup={"item_id": "testimage"})
            self.assertEqual(refs.count(), 4)
            for ref in refs:
                self.assertEqual(ref.get("published"), False)
            CropService().update_media_references(item, original, True)
            refs = references_service.get(req=None, lookup={"item_id": "testimage"})
            for ref in refs:
                self.assertEqual(ref.get("published"), True)

            remove_media_files(item)
            self.assertEqual(0, media_delete.call_count)

            item = {"_id": "testimage2", "type": "picture", "renditions": self.media}

            original = item.copy()
            CropService().update_media_references(item, original)
            references_service = get_resource_service("media_references")
            refs = references_service.get(req=None, lookup={"item_id": "testimage2"})
            self.assertEqual(refs.count(), 4)
            for ref in refs:
                self.assertEqual(ref.get("published"), False)

            remove_media_files(item)
            self.assertEqual(0, media_delete.call_count)

            item = {
                "_id": "testimage3",
                "type": "picture",
                "renditions": {
                    "viewImage": {
                        "media": "123",
                        "mimetype": "image/jpeg",
                        "href": "http://192.168.220.209/api/upload/abc/raw?_schema=http",
                        "height": 452,
                        "width": 640,
                    },
                    "thumbnail": {
                        "media": "456",
                        "mimetype": "image/jpeg",
                        "href": "http://192.168.220.209/api/upload/abc/raw?_schema=http",
                        "height": 120,
                        "width": 169,
                    },
                },
            }

            original = item.copy()
            CropService().update_media_references(item, original)
            references_service = get_resource_service("media_references")
            refs = references_service.get(req=None, lookup={"item_id": "testimage3"})
            self.assertEqual(refs.count(), 2)
            for ref in refs:
                self.assertEqual(ref.get("published"), False)

            remove_media_files(item)
            self.assertEqual(2, media_delete.call_count)
            for key, rendition in item.get("renditions").items():
                media_delete.assert_any_call(rendition["media"])

    def test_remove_media_files_for_picture_associations(self):
        item = {
            "_id": "testimage",
            "type": "text",
            "associations": {
                "featuremedia": {"_id": "123", "renditions": self.media},
                "featurevideo": {
                    "_id": "456",
                    "renditions": {
                        "viewImage": {
                            "media": "testing_123",
                            "mimetype": "image/jpeg",
                            "href": "http://192.168.220.209/api/upload/abc/raw?_schema=http",
                            "height": 452,
                            "width": 640,
                        },
                        "thumbnail": {
                            "media": "testing_456",
                            "mimetype": "image/jpeg",
                            "href": "http://192.168.220.209/api/upload/abc/raw?_schema=http",
                            "height": 120,
                            "width": 169,
                        },
                    },
                },
            },
        }

        original = item.copy()
        with patch.object(self.app.media, "delete") as media_delete:
            CropService().update_media_references(item, original)
            references_service = get_resource_service("media_references")
            refs = references_service.get(req=None, lookup={"item_id": "testimage"})
            self.assertEqual(refs.count(), 6)
            for ref in refs:
                self.assertEqual(ref.get("published"), False)
            CropService().update_media_references(item, original, True)
            refs = references_service.get(req=None, lookup={"item_id": "testimage"})
            for ref in refs:
                self.assertEqual(ref.get("published"), True)

            remove_media_files(item)
            self.assertEqual(0, media_delete.call_count)

    def test_remove_media_files_for_attachments(self):
        attachments = self.app.data.insert("attachments", [{"media": "foo"}])
        item = {
            "_id": "test",
            "type": "text",
            "attachments": [
                {"attachment": attachments[0]},
            ],
        }
        with patch.object(self.app.media, "delete") as media_delete:
            remove_media_files(item)
        media_delete.assert_any_call("foo", "attachments")

    def test_delete_by_ids(self):
        ids = self.app.data.insert(ARCHIVE, self.articles)
        archive_service = get_resource_service(ARCHIVE)
        archive_service.on_delete = MagicMock()
        archive_service.delete_by_article_ids(ids)
        self.assertTrue(self.app.data.mongo.is_empty(ARCHIVE))
        self.assertTrue(self.app.data.elastic.is_empty(ARCHIVE))
        self.assertEqual(len(self.articles), archive_service.on_delete.call_count)

    def test_remove_renditions_from_all_versions(self):
        renditions = copy.copy(self.media)

        ids = self.app.data.insert(
            ARCHIVE,
            [
                {
                    "state": "spiked",
                    "expiry": get_expiry_date(-10),
                    "type": "picture",
                    "renditions": {},
                }
            ],
        )

        self.app.data.insert(
            "archive_versions",
            [
                {
                    "_id_document": ids[0],
                    "type": "picture",
                    "renditions": renditions,
                }
            ],
        )

        with patch.object(self.app.media, "delete") as media_delete:
            get_resource_service("archive").delete_by_article_ids(ids)
            for key, rendition in renditions.items():
                media_delete.assert_any_call(rendition["media"])

    def _get_original(self, _id):
        return self.app.data.find_one(ARCHIVE, None, _id=_id)
